/****************************************************************************
** Meta object code from reading C++ file 'abstractbarchartitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/barchart/abstractbarchartitem_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractbarchartitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20AbstractBarChartItemE_t {};
} // unnamed namespace

template <> constexpr inline auto AbstractBarChartItem::qt_create_metaobjectdata<qt_meta_tag_ZN20AbstractBarChartItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AbstractBarChartItem",
        "handleDomainUpdated",
        "",
        "handleLayoutChanged",
        "handleLabelsVisibleChanged",
        "visible",
        "handleDataStructureChanged",
        "handleVisibleChanged",
        "handleOpacityChanged",
        "handleUpdatedBars",
        "handleLabelsPositionChanged",
        "positionLabels",
        "handleBarValueChange",
        "index",
        "QBarSet*",
        "barset",
        "handleBarValueAdd",
        "count",
        "handleBarValueRemove",
        "handleSeriesAdded",
        "QAbstractSeries*",
        "series",
        "handleSeriesRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'handleDomainUpdated'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleLayoutChanged'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleLabelsVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'handleDataStructureChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleVisibleChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleOpacityChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleUpdatedBars'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleLabelsPositionChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'positionLabels'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleBarValueChange'
        QtMocHelpers::SlotData<void(int, QBarSet *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleBarValueAdd'
        QtMocHelpers::SlotData<void(int, int, QBarSet *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::Int, 17 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleBarValueRemove'
        QtMocHelpers::SlotData<void(int, int, QBarSet *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 }, { QMetaType::Int, 17 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleSeriesAdded'
        QtMocHelpers::SlotData<void(QAbstractSeries *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'handleSeriesRemoved'
        QtMocHelpers::SlotData<void(QAbstractSeries *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractBarChartItem, qt_meta_tag_ZN20AbstractBarChartItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AbstractBarChartItem::staticMetaObject = { {
    QMetaObject::SuperData::link<ChartItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20AbstractBarChartItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20AbstractBarChartItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20AbstractBarChartItemE_t>.metaTypes,
    nullptr
} };

void AbstractBarChartItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractBarChartItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->handleDomainUpdated(); break;
        case 1: _t->handleLayoutChanged(); break;
        case 2: _t->handleLabelsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->handleDataStructureChanged(); break;
        case 4: _t->handleVisibleChanged(); break;
        case 5: _t->handleOpacityChanged(); break;
        case 6: _t->handleUpdatedBars(); break;
        case 7: _t->handleLabelsPositionChanged(); break;
        case 8: _t->positionLabels(); break;
        case 9: _t->handleBarValueChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 10: _t->handleBarValueAdd((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[3]))); break;
        case 11: _t->handleBarValueRemove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[3]))); break;
        case 12: _t->handleSeriesAdded((*reinterpret_cast<std::add_pointer_t<QAbstractSeries*>>(_a[1]))); break;
        case 13: _t->handleSeriesRemoved((*reinterpret_cast<std::add_pointer_t<QAbstractSeries*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSeries* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSeries* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *AbstractBarChartItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbstractBarChartItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20AbstractBarChartItemE_t>.strings))
        return static_cast<void*>(this);
    return ChartItem::qt_metacast(_clname);
}

int AbstractBarChartItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChartItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
