/****************************************************************************
** Meta object code from reading C++ file 'qabstractbarseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/barchart/qabstractbarseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractbarseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QAbstractBarSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractBarSeries::qt_create_metaobjectdata<qt_meta_tag_ZN18QAbstractBarSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractBarSeries",
        "clicked",
        "",
        "index",
        "QBarSet*",
        "barset",
        "hovered",
        "status",
        "pressed",
        "released",
        "doubleClicked",
        "countChanged",
        "labelsVisibleChanged",
        "labelsFormatChanged",
        "format",
        "labelsPositionChanged",
        "QAbstractBarSeries::LabelsPosition",
        "position",
        "labelsAngleChanged",
        "angle",
        "labelsPrecisionChanged",
        "precision",
        "barsetsAdded",
        "QList<QBarSet*>",
        "sets",
        "barsetsRemoved",
        "barWidth",
        "count",
        "labelsVisible",
        "labelsFormat",
        "labelsPosition",
        "LabelsPosition",
        "labelsAngle",
        "labelsPrecision",
        "LabelsCenter",
        "LabelsInsideEnd",
        "LabelsInsideBase",
        "LabelsOutsideEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(int, QBarSet *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool, int, QBarSet *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(int, QBarSet *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(int, QBarSet *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(int, QBarSet *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelsVisibleChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelsFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'labelsPositionChanged'
        QtMocHelpers::SignalData<void(QAbstractBarSeries::LabelsPosition)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'labelsAngleChanged'
        QtMocHelpers::SignalData<void(qreal)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 19 },
        }}),
        // Signal 'labelsPrecisionChanged'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Signal 'barsetsAdded'
        QtMocHelpers::SignalData<void(const QList<QBarSet*> &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'barsetsRemoved'
        QtMocHelpers::SignalData<void(const QList<QBarSet*> &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'barWidth'
        QtMocHelpers::PropertyData<qreal>(26, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'count'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'labelsFormat'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'labelsPosition'
        QtMocHelpers::PropertyData<enum LabelsPosition>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'labelsAngle'
        QtMocHelpers::PropertyData<qreal>(32, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'labelsPrecision'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LabelsPosition'
        QtMocHelpers::EnumData<enum LabelsPosition>(31, 31, QMC::EnumFlags{}).add({
            {   34, LabelsPosition::LabelsCenter },
            {   35, LabelsPosition::LabelsInsideEnd },
            {   36, LabelsPosition::LabelsInsideBase },
            {   37, LabelsPosition::LabelsOutsideEnd },
        }),
    };
    return QtMocHelpers::metaObjectData<QAbstractBarSeries, qt_meta_tag_ZN18QAbstractBarSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstractBarSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QAbstractBarSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QAbstractBarSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QAbstractBarSeriesE_t>.metaTypes,
    nullptr
} };

void QAbstractBarSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractBarSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[3]))); break;
        case 2: _t->pressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 3: _t->released((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBarSet*>>(_a[2]))); break;
        case 5: _t->countChanged(); break;
        case 6: _t->labelsVisibleChanged(); break;
        case 7: _t->labelsFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->labelsPositionChanged((*reinterpret_cast<std::add_pointer_t<QAbstractBarSeries::LabelsPosition>>(_a[1]))); break;
        case 9: _t->labelsAngleChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->labelsPrecisionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->barsetsAdded((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1]))); break;
        case 12: _t->barsetsRemoved((*reinterpret_cast<std::add_pointer_t<QList<QBarSet*>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(int , QBarSet * )>(_a, &QAbstractBarSeries::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(bool , int , QBarSet * )>(_a, &QAbstractBarSeries::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(int , QBarSet * )>(_a, &QAbstractBarSeries::pressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(int , QBarSet * )>(_a, &QAbstractBarSeries::released, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(int , QBarSet * )>(_a, &QAbstractBarSeries::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)()>(_a, &QAbstractBarSeries::countChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)()>(_a, &QAbstractBarSeries::labelsVisibleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(const QString & )>(_a, &QAbstractBarSeries::labelsFormatChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(QAbstractBarSeries::LabelsPosition )>(_a, &QAbstractBarSeries::labelsPositionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(qreal )>(_a, &QAbstractBarSeries::labelsAngleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(int )>(_a, &QAbstractBarSeries::labelsPrecisionChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(const QList<QBarSet*> & )>(_a, &QAbstractBarSeries::barsetsAdded, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractBarSeries::*)(const QList<QBarSet*> & )>(_a, &QAbstractBarSeries::barsetsRemoved, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->barWidth(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isLabelsVisible(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->labelsFormat(); break;
        case 4: *reinterpret_cast<enum LabelsPosition*>(_v) = _t->labelsPosition(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->labelsAngle(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->labelsPrecision(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBarWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setLabelsFormat(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setLabelsPosition(*reinterpret_cast<enum LabelsPosition*>(_v)); break;
        case 5: _t->setLabelsAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setLabelsPrecision(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractBarSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractBarSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QAbstractBarSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QAbstractBarSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QAbstractBarSeries::clicked(int _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QAbstractBarSeries::hovered(bool _t1, int _t2, QBarSet * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void QAbstractBarSeries::pressed(int _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QAbstractBarSeries::released(int _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QAbstractBarSeries::doubleClicked(int _t1, QBarSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QAbstractBarSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QAbstractBarSeries::labelsVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QAbstractBarSeries::labelsFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstractBarSeries::labelsPositionChanged(QAbstractBarSeries::LabelsPosition _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstractBarSeries::labelsAngleChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstractBarSeries::labelsPrecisionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstractBarSeries::barsetsAdded(const QList<QBarSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstractBarSeries::barsetsRemoved(const QList<QBarSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
