/****************************************************************************
** Meta object code from reading C++ file 'declarativecandlestickseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/chartsqml2/declarativecandlestickseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativecandlestickseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25DeclarativeCandlestickSetE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeCandlestickSet::qt_create_metaobjectdata<qt_meta_tag_ZN25DeclarativeCandlestickSetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeCandlestickSet",
        "QML.Element",
        "CandlestickSet",
        "QML.AddedInVersion",
        "514",
        "brushFilenameChanged",
        "",
        "brushFilename",
        "handleBrushChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'brushFilenameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'handleBrushChanged'
        QtMocHelpers::SlotData<void()>(8, 6, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'brushFilename'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeCandlestickSet, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeCandlestickSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QCandlestickSet::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25DeclarativeCandlestickSetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25DeclarativeCandlestickSetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25DeclarativeCandlestickSetE_t>.metaTypes,
    nullptr
} };

void DeclarativeCandlestickSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeCandlestickSet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->brushFilenameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->handleBrushChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSet::*)(const QString & )>(_a, &DeclarativeCandlestickSet::brushFilenameChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->brushFilename(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBrushFilename(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeCandlestickSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeCandlestickSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25DeclarativeCandlestickSetE_t>.strings))
        return static_cast<void*>(this);
    return QCandlestickSet::qt_metacast(_clname);
}

int DeclarativeCandlestickSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCandlestickSet::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeCandlestickSet::brushFilenameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeCandlestickSeries::qt_create_metaobjectdata<qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeCandlestickSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "CandlestickSeries",
        "QML.AddedInVersion",
        "514",
        "axisXChanged",
        "",
        "QAbstractAxis*",
        "axis",
        "axisYChanged",
        "axisXTopChanged",
        "axisYRightChanged",
        "clicked",
        "DeclarativeCandlestickSet*",
        "set",
        "hovered",
        "status",
        "pressed",
        "released",
        "doubleClicked",
        "brushFilenameChanged",
        "brushFilename",
        "onClicked",
        "QCandlestickSet*",
        "onHovered",
        "onPressed",
        "onReleased",
        "onDoubleClicked",
        "handleBrushChanged",
        "at",
        "index",
        "append",
        "remove",
        "open",
        "high",
        "low",
        "close",
        "timestamp",
        "insert",
        "clear",
        "axisX",
        "axisY",
        "axisXTop",
        "axisYRight",
        "QQmlListProperty<QObject>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'axisXTopChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'axisYRightChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(DeclarativeCandlestickSet *)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool, DeclarativeCandlestickSet *)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 }, { 0x80000000 | 15, 16 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(DeclarativeCandlestickSet *)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(DeclarativeCandlestickSet *)>(20, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(DeclarativeCandlestickSet *)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'brushFilenameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'onClicked'
        QtMocHelpers::SlotData<void(QCandlestickSet *)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 16 },
        }}),
        // Slot 'onHovered'
        QtMocHelpers::SlotData<void(bool, QCandlestickSet *)>(26, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 }, { 0x80000000 | 25, 16 },
        }}),
        // Slot 'onPressed'
        QtMocHelpers::SlotData<void(QCandlestickSet *)>(27, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 16 },
        }}),
        // Slot 'onReleased'
        QtMocHelpers::SlotData<void(QCandlestickSet *)>(28, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 16 },
        }}),
        // Slot 'onDoubleClicked'
        QtMocHelpers::SlotData<void(QCandlestickSet *)>(29, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 16 },
        }}),
        // Slot 'handleBrushChanged'
        QtMocHelpers::SlotData<void()>(30, 8, QMC::AccessPrivate, QMetaType::Void),
        // Method 'at'
        QtMocHelpers::MethodData<DeclarativeCandlestickSet *(int)>(31, 8, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::Int, 32 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<bool(DeclarativeCandlestickSet *)>(33, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(DeclarativeCandlestickSet *)>(34, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<bool(qreal, qreal, qreal, qreal, qreal)>(33, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 35 }, { QMetaType::QReal, 36 }, { QMetaType::QReal, 37 }, { QMetaType::QReal, 38 },
            { QMetaType::QReal, 39 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(qreal)>(34, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 39 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<bool(int, DeclarativeCandlestickSet *)>(40, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 32 }, { 0x80000000 | 15, 16 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(41, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'axisX'
        QtMocHelpers::PropertyData<QAbstractAxis*>(42, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'axisY'
        QtMocHelpers::PropertyData<QAbstractAxis*>(43, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'axisXTop'
        QtMocHelpers::PropertyData<QAbstractAxis*>(44, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'axisYRight'
        QtMocHelpers::PropertyData<QAbstractAxis*>(45, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'brushFilename'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeCandlestickSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeCandlestickSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QCandlestickSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeCandlestickSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeCandlestickSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 1: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 2: _t->axisXTopChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 3: _t->axisYRightChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 4: _t->clicked((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1]))); break;
        case 5: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[2]))); break;
        case 6: _t->pressed((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1]))); break;
        case 7: _t->released((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1]))); break;
        case 8: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1]))); break;
        case 9: _t->brushFilenameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->onClicked((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 11: _t->onHovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 12: _t->onPressed((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 13: _t->onReleased((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 14: _t->onDoubleClicked((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 15: _t->handleBrushChanged(); break;
        case 16: { DeclarativeCandlestickSet* _r = _t->at((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DeclarativeCandlestickSet**>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->append((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[5])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->insert((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeclarativeCandlestickSet*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCandlestickSet* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCandlestickSet* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCandlestickSet* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCandlestickSet* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCandlestickSet* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeCandlestickSet* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(QAbstractAxis * )>(_a, &DeclarativeCandlestickSeries::axisXChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(QAbstractAxis * )>(_a, &DeclarativeCandlestickSeries::axisYChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(QAbstractAxis * )>(_a, &DeclarativeCandlestickSeries::axisXTopChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(QAbstractAxis * )>(_a, &DeclarativeCandlestickSeries::axisYRightChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(DeclarativeCandlestickSet * )>(_a, &DeclarativeCandlestickSeries::clicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(bool , DeclarativeCandlestickSet * )>(_a, &DeclarativeCandlestickSeries::hovered, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(DeclarativeCandlestickSet * )>(_a, &DeclarativeCandlestickSeries::pressed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(DeclarativeCandlestickSet * )>(_a, &DeclarativeCandlestickSeries::released, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(DeclarativeCandlestickSet * )>(_a, &DeclarativeCandlestickSeries::doubleClicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCandlestickSeries::*)(const QString & )>(_a, &DeclarativeCandlestickSeries::brushFilenameChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisX(); break;
        case 1: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisY(); break;
        case 2: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisXTop(); break;
        case 3: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisYRight(); break;
        case 4: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->brushFilename(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAxisX(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 1: _t->setAxisY(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 2: _t->setAxisXTop(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 3: _t->setAxisYRight(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 5: _t->setBrushFilename(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeCandlestickSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeCandlestickSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28DeclarativeCandlestickSeriesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QCandlestickSeries::qt_metacast(_clname);
}

int DeclarativeCandlestickSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCandlestickSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeCandlestickSeries::axisXChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeCandlestickSeries::axisYChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeCandlestickSeries::axisXTopChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeclarativeCandlestickSeries::axisYRightChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DeclarativeCandlestickSeries::clicked(DeclarativeCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DeclarativeCandlestickSeries::hovered(bool _t1, DeclarativeCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void DeclarativeCandlestickSeries::pressed(DeclarativeCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DeclarativeCandlestickSeries::released(DeclarativeCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DeclarativeCandlestickSeries::doubleClicked(DeclarativeCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DeclarativeCandlestickSeries::brushFilenameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
