/****************************************************************************
** Meta object code from reading C++ file 'declarativelineseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/chartsqml2/declarativelineseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativelineseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DeclarativeLineSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeLineSeries::qt_create_metaobjectdata<qt_meta_tag_ZN21DeclarativeLineSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeLineSeries",
        "DefaultProperty",
        "declarativeChildren",
        "QML.Element",
        "LineSeries",
        "QML.AddedInVersion",
        "256",
        "QML.ExtraVersion",
        "512",
        "countChanged",
        "",
        "count",
        "axisXChanged",
        "QAbstractAxis*",
        "axis",
        "axisYChanged",
        "axisXTopChanged",
        "axisYRightChanged",
        "axisAngularChanged",
        "axisRadialChanged",
        "widthChanged",
        "width",
        "styleChanged",
        "Qt::PenStyle",
        "style",
        "capStyleChanged",
        "Qt::PenCapStyle",
        "capStyle",
        "handleCountChanged",
        "index",
        "append",
        "x",
        "y",
        "replace",
        "oldX",
        "oldY",
        "newX",
        "newY",
        "remove",
        "removePoints",
        "insert",
        "clear",
        "at",
        "QPointF",
        "axisX",
        "axisY",
        "axisXTop",
        "axisYRight",
        "axisAngular",
        "axisRadial",
        "QQmlListProperty<QObject>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void(int)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'axisXChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(12, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(15, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'axisXTopChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(16, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'axisYRightChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(17, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'axisAngularChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(18, 10, QMC::AccessPublic, 0x103, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'axisRadialChanged'
        QtMocHelpers::RevisionedSignalData<void(QAbstractAxis *)>(19, 10, QMC::AccessPublic, 0x103, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'widthChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(20, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QReal, 21 },
        }}),
        // Signal 'styleChanged'
        QtMocHelpers::RevisionedSignalData<void(Qt::PenStyle)>(22, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'capStyleChanged'
        QtMocHelpers::RevisionedSignalData<void(Qt::PenCapStyle)>(25, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'handleCountChanged'
        QtMocHelpers::SlotData<void(int)>(28, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(qreal, qreal)>(30, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 31 }, { QMetaType::QReal, 32 },
        }}),
        // Method 'replace'
        QtMocHelpers::MethodData<void(qreal, qreal, qreal, qreal)>(33, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 34 }, { QMetaType::QReal, 35 }, { QMetaType::QReal, 36 }, { QMetaType::QReal, 37 },
        }}),
        // Method 'replace'
        QtMocHelpers::RevisionedMethodData<void(int, qreal, qreal)>(33, 10, QMC::AccessPublic, 0x103, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { QMetaType::QReal, 36 }, { QMetaType::QReal, 37 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void(qreal, qreal)>(38, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 31 }, { QMetaType::QReal, 32 },
        }}),
        // Method 'remove'
        QtMocHelpers::RevisionedMethodData<void(int)>(38, 10, QMC::AccessPublic, 0x103, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Method 'removePoints'
        QtMocHelpers::RevisionedMethodData<void(int, int)>(39, 10, QMC::AccessPublic, 0x201, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { QMetaType::Int, 11 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<void(int, qreal, qreal)>(40, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 }, { QMetaType::QReal, 31 }, { QMetaType::QReal, 32 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(41, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'at'
        QtMocHelpers::MethodData<QPointF(int)>(42, 10, QMC::AccessPublic, 0x80000000 | 43, {{
            { QMetaType::Int, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'axisX'
        QtMocHelpers::PropertyData<QAbstractAxis*>(44, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1, 0x101),
        // property 'axisY'
        QtMocHelpers::PropertyData<QAbstractAxis*>(45, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2, 0x101),
        // property 'axisXTop'
        QtMocHelpers::PropertyData<QAbstractAxis*>(46, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3, 0x102),
        // property 'axisYRight'
        QtMocHelpers::PropertyData<QAbstractAxis*>(47, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4, 0x102),
        // property 'axisAngular'
        QtMocHelpers::PropertyData<QAbstractAxis*>(48, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5, 0x103),
        // property 'axisRadial'
        QtMocHelpers::PropertyData<QAbstractAxis*>(49, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6, 0x103),
        // property 'width'
        QtMocHelpers::PropertyData<qreal>(21, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7, 0x101),
        // property 'style'
        QtMocHelpers::PropertyData<Qt::PenStyle>(24, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8, 0x101),
        // property 'capStyle'
        QtMocHelpers::PropertyData<Qt::PenCapStyle>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9, 0x101),
        // property 'declarativeChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeLineSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeLineSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DeclarativeLineSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DeclarativeLineSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DeclarativeLineSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeLineSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeLineSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 2: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 3: _t->axisXTopChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 4: _t->axisYRightChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 5: _t->axisAngularChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 6: _t->axisRadialChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 7: _t->widthChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 8: _t->styleChanged((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 9: _t->capStyleChanged((*reinterpret_cast<std::add_pointer_t<Qt::PenCapStyle>>(_a[1]))); break;
        case 10: _t->handleCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->append((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 12: _t->replace((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4]))); break;
        case 13: _t->replace((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 14: _t->remove((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 15: _t->remove((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->removePoints((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->insert((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3]))); break;
        case 18: _t->clear(); break;
        case 19: { QPointF _r = _t->at((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(int )>(_a, &DeclarativeLineSeries::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisXChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisYChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisXTopChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisYRightChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisAngularChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(QAbstractAxis * )>(_a, &DeclarativeLineSeries::axisRadialChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(qreal )>(_a, &DeclarativeLineSeries::widthChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(Qt::PenStyle )>(_a, &DeclarativeLineSeries::styleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeLineSeries::*)(Qt::PenCapStyle )>(_a, &DeclarativeLineSeries::capStyleChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
        case 5:
        case 4:
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisX(); break;
        case 2: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisY(); break;
        case 3: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisXTop(); break;
        case 4: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisYRight(); break;
        case 5: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisAngular(); break;
        case 6: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisRadial(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->width(); break;
        case 8: *reinterpret_cast<Qt::PenStyle*>(_v) = _t->style(); break;
        case 9: *reinterpret_cast<Qt::PenCapStyle*>(_v) = _t->capStyle(); break;
        case 10: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->declarativeChildren(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setAxisX(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 2: _t->setAxisY(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 3: _t->setAxisXTop(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 4: _t->setAxisYRight(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 5: _t->setAxisAngular(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 6: _t->setAxisRadial(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 7: _t->setWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 8: _t->setStyle(*reinterpret_cast<Qt::PenStyle*>(_v)); break;
        case 9: _t->setCapStyle(*reinterpret_cast<Qt::PenCapStyle*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeLineSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeLineSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DeclarativeLineSeriesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DeclarativeXySeries"))
        return static_cast< DeclarativeXySeries*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QLineSeries::qt_metacast(_clname);
}

int DeclarativeLineSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeLineSeries::countChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeLineSeries::axisXChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeLineSeries::axisYChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeclarativeLineSeries::axisXTopChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DeclarativeLineSeries::axisYRightChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DeclarativeLineSeries::axisAngularChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DeclarativeLineSeries::axisRadialChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DeclarativeLineSeries::widthChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DeclarativeLineSeries::styleChanged(Qt::PenStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DeclarativeLineSeries::capStyleChanged(Qt::PenCapStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
