/****************************************************************************
** Meta object code from reading C++ file 'cartesianchartaxis_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/axis/cartesianchartaxis_p.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cartesianchartaxis_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18CartesianChartAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto CartesianChartAxis::qt_create_metaobjectdata<qt_meta_tag_ZN18CartesianChartAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CartesianChartAxis",
        "handleArrowPenChanged",
        "",
        "QPen",
        "pen",
        "handleGridPenChanged",
        "handleShadesBrushChanged",
        "QBrush",
        "brush",
        "handleShadesPenChanged",
        "handleMinorArrowPenChanged",
        "handleMinorGridPenChanged",
        "handleGridLineColorChanged",
        "QColor",
        "color",
        "handleMinorGridLineColorChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'handleArrowPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'handleGridPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'handleShadesBrushChanged'
        QtMocHelpers::SlotData<void(const QBrush &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'handleShadesPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'handleMinorArrowPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'handleMinorGridPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'handleGridLineColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'handleMinorGridLineColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CartesianChartAxis, qt_meta_tag_ZN18CartesianChartAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CartesianChartAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<ChartAxisElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CartesianChartAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CartesianChartAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18CartesianChartAxisE_t>.metaTypes,
    nullptr
} };

void CartesianChartAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CartesianChartAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->handleArrowPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 1: _t->handleGridPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 2: _t->handleShadesBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 3: _t->handleShadesPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 4: _t->handleMinorArrowPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 5: _t->handleMinorGridPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 6: _t->handleGridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->handleMinorGridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CartesianChartAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CartesianChartAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CartesianChartAxisE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsLayoutItem"))
        return static_cast< QGraphicsLayoutItem*>(this);
    return ChartAxisElement::qt_metacast(_clname);
}

int CartesianChartAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChartAxisElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
