/****************************************************************************
** Meta object code from reading C++ file 'qhboxplotmodelmapper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/boxplotchart/qhboxplotmodelmapper.h"
#include <QtCharts/qboxplotseries.h>
#include <QtCharts/qboxplotseries.h>
#include <QtCore/qabstractitemmodel.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qhboxplotmodelmapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QHBoxPlotModelMapperE_t {};
} // unnamed namespace

template <> constexpr inline auto QHBoxPlotModelMapper::qt_create_metaobjectdata<qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHBoxPlotModelMapper",
        "seriesReplaced",
        "",
        "modelReplaced",
        "firstBoxSetRowChanged",
        "lastBoxSetRowChanged",
        "firstColumnChanged",
        "columnCountChanged",
        "series",
        "QBoxPlotSeries*",
        "model",
        "QAbstractItemModel*",
        "firstBoxSetRow",
        "lastBoxSetRow",
        "firstColumn",
        "columnCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'seriesReplaced'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelReplaced'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firstBoxSetRowChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastBoxSetRowChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firstColumnChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnCountChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'series'
        QtMocHelpers::PropertyData<QBoxPlotSeries*>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'model'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'firstBoxSetRow'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'lastBoxSetRow'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'firstColumn'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'columnCount'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QHBoxPlotModelMapper, qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHBoxPlotModelMapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QBoxPlotModelMapper::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>.metaTypes,
    nullptr
} };

void QHBoxPlotModelMapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHBoxPlotModelMapper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->seriesReplaced(); break;
        case 1: _t->modelReplaced(); break;
        case 2: _t->firstBoxSetRowChanged(); break;
        case 3: _t->lastBoxSetRowChanged(); break;
        case 4: _t->firstColumnChanged(); break;
        case 5: _t->columnCountChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::seriesReplaced, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::modelReplaced, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::firstBoxSetRowChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::lastBoxSetRowChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::firstColumnChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHBoxPlotModelMapper::*)()>(_a, &QHBoxPlotModelMapper::columnCountChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QBoxPlotSeries**>(_v) = _t->series(); break;
        case 1: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->model(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->firstBoxSetRow(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->lastBoxSetRow(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->firstColumn(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->columnCount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSeries(*reinterpret_cast<QBoxPlotSeries**>(_v)); break;
        case 1: _t->setModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 2: _t->setFirstBoxSetRow(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setLastBoxSetRow(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setFirstColumn(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setColumnCount(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QHBoxPlotModelMapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHBoxPlotModelMapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QHBoxPlotModelMapperE_t>.strings))
        return static_cast<void*>(this);
    return QBoxPlotModelMapper::qt_metacast(_clname);
}

int QHBoxPlotModelMapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QBoxPlotModelMapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QHBoxPlotModelMapper::seriesReplaced()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QHBoxPlotModelMapper::modelReplaced()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QHBoxPlotModelMapper::firstBoxSetRowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QHBoxPlotModelMapper::lastBoxSetRowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QHBoxPlotModelMapper::firstColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QHBoxPlotModelMapper::columnCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
