/****************************************************************************
** Meta object code from reading C++ file 'qpieseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/piechart/qpieseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpieseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17QPieSeriesPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto QPieSeriesPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN17QPieSeriesPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPieSeriesPrivate",
        "calculatedDataChanged",
        "",
        "pieSizeChanged",
        "pieStartAngleChanged",
        "pieEndAngleChanged",
        "horizontalPositionChanged",
        "verticalPositionChanged",
        "sliceValueChanged",
        "sliceClicked",
        "sliceHovered",
        "state",
        "slicePressed",
        "sliceReleased",
        "sliceDoubleClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'calculatedDataChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pieSizeChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pieStartAngleChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pieEndAngleChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'horizontalPositionChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'verticalPositionChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sliceValueChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sliceClicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sliceHovered'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'slicePressed'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sliceReleased'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sliceDoubleClicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPieSeriesPrivate, qt_meta_tag_ZN17QPieSeriesPrivateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPieSeriesPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeriesPrivate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QPieSeriesPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QPieSeriesPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QPieSeriesPrivateE_t>.metaTypes,
    nullptr
} };

void QPieSeriesPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPieSeriesPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->calculatedDataChanged(); break;
        case 1: _t->pieSizeChanged(); break;
        case 2: _t->pieStartAngleChanged(); break;
        case 3: _t->pieEndAngleChanged(); break;
        case 4: _t->horizontalPositionChanged(); break;
        case 5: _t->verticalPositionChanged(); break;
        case 6: _t->sliceValueChanged(); break;
        case 7: _t->sliceClicked(); break;
        case 8: _t->sliceHovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slicePressed(); break;
        case 10: _t->sliceReleased(); break;
        case 11: _t->sliceDoubleClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::calculatedDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::pieSizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::pieStartAngleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::pieEndAngleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::horizontalPositionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeriesPrivate::*)()>(_a, &QPieSeriesPrivate::verticalPositionChanged, 5))
            return;
    }
}

const QMetaObject *QPieSeriesPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPieSeriesPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QPieSeriesPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeriesPrivate::qt_metacast(_clname);
}

int QPieSeriesPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeriesPrivate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QPieSeriesPrivate::calculatedDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QPieSeriesPrivate::pieSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QPieSeriesPrivate::pieStartAngleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QPieSeriesPrivate::pieEndAngleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QPieSeriesPrivate::horizontalPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QPieSeriesPrivate::verticalPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
