/****************************************************************************
** Meta object code from reading C++ file 'qlegendmarker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/legend/qlegendmarker.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlegendmarker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QLegendMarkerE_t {};
} // unnamed namespace

template <> constexpr inline auto QLegendMarker::qt_create_metaobjectdata<qt_meta_tag_ZN13QLegendMarkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLegendMarker",
        "clicked",
        "",
        "hovered",
        "status",
        "labelChanged",
        "labelBrushChanged",
        "fontChanged",
        "penChanged",
        "brushChanged",
        "visibleChanged",
        "shapeChanged",
        "label",
        "labelBrush",
        "QBrush",
        "font",
        "QFont",
        "pen",
        "QPen",
        "brush",
        "visible",
        "shape",
        "QLegend::MarkerShape",
        "LegendMarkerType",
        "LegendMarkerTypeArea",
        "LegendMarkerTypeBar",
        "LegendMarkerTypePie",
        "LegendMarkerTypeXY",
        "LegendMarkerTypeBoxPlot",
        "LegendMarkerTypeCandlestick"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelBrushChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'penChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'brushChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shapeChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'label'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'labelBrush'
        QtMocHelpers::PropertyData<QBrush>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'font'
        QtMocHelpers::PropertyData<QFont>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'pen'
        QtMocHelpers::PropertyData<QPen>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'brush'
        QtMocHelpers::PropertyData<QBrush>(19, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'shape'
        QtMocHelpers::PropertyData<QLegend::MarkerShape>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LegendMarkerType'
        QtMocHelpers::EnumData<enum LegendMarkerType>(23, 23, QMC::EnumFlags{}).add({
            {   24, LegendMarkerType::LegendMarkerTypeArea },
            {   25, LegendMarkerType::LegendMarkerTypeBar },
            {   26, LegendMarkerType::LegendMarkerTypePie },
            {   27, LegendMarkerType::LegendMarkerTypeXY },
            {   28, LegendMarkerType::LegendMarkerTypeBoxPlot },
            {   29, LegendMarkerType::LegendMarkerTypeCandlestick },
        }),
    };
    return QtMocHelpers::metaObjectData<QLegendMarker, qt_meta_tag_ZN13QLegendMarkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13QLegendMarkerE[] = {
    QMetaObject::SuperData::link<QLegend::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QLegendMarker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QLegendMarkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QLegendMarkerE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13QLegendMarkerE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QLegendMarkerE_t>.metaTypes,
    nullptr
} };

void QLegendMarker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLegendMarker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked(); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->labelChanged(); break;
        case 3: _t->labelBrushChanged(); break;
        case 4: _t->fontChanged(); break;
        case 5: _t->penChanged(); break;
        case 6: _t->brushChanged(); break;
        case 7: _t->visibleChanged(); break;
        case 8: _t->shapeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)(bool )>(_a, &QLegendMarker::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::labelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::labelBrushChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::fontChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::penChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::brushChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::visibleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegendMarker::*)()>(_a, &QLegendMarker::shapeChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 1: *reinterpret_cast<QBrush*>(_v) = _t->labelBrush(); break;
        case 2: *reinterpret_cast<QFont*>(_v) = _t->font(); break;
        case 3: *reinterpret_cast<QPen*>(_v) = _t->pen(); break;
        case 4: *reinterpret_cast<QBrush*>(_v) = _t->brush(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 6: *reinterpret_cast<QLegend::MarkerShape*>(_v) = _t->shape(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLabelBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 2: _t->setFont(*reinterpret_cast<QFont*>(_v)); break;
        case 3: _t->setPen(*reinterpret_cast<QPen*>(_v)); break;
        case 4: _t->setBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 5: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setShape(*reinterpret_cast<QLegend::MarkerShape*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QLegendMarker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLegendMarker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QLegendMarkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLegendMarker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QLegendMarker::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QLegendMarker::hovered(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QLegendMarker::labelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QLegendMarker::labelBrushChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QLegendMarker::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QLegendMarker::penChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QLegendMarker::brushChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QLegendMarker::visibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QLegendMarker::shapeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
