/****************************************************************************
** Meta object code from reading C++ file 'declarativeboxplotseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/chartsqml2/declarativeboxplotseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativeboxplotseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17DeclarativeBoxSetE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeBoxSet::qt_create_metaobjectdata<qt_meta_tag_ZN17DeclarativeBoxSetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeBoxSet",
        "QML.Element",
        "BoxSet",
        "QML.AddedInVersion",
        "259",
        "QML.ExtraVersion",
        "512",
        "changedValues",
        "",
        "changedValue",
        "index",
        "brushFilenameChanged",
        "brushFilename",
        "handleBrushChanged",
        "append",
        "value",
        "clear",
        "at",
        "setValue",
        "values",
        "QVariantList",
        "label",
        "count",
        "ValuePositions",
        "LowerExtreme",
        "LowerQuartile",
        "Median",
        "UpperQuartile",
        "UpperExtreme"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changedValues'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'changedValue'
        QtMocHelpers::SignalData<void(int)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Signal 'brushFilenameChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(11, 8, QMC::AccessPublic, 0x104, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'handleBrushChanged'
        QtMocHelpers::SlotData<void()>(13, 8, QMC::AccessPrivate, QMetaType::Void),
        // Method 'append'
        QtMocHelpers::MethodData<void(qreal)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'at'
        QtMocHelpers::MethodData<qreal(int)>(17, 8, QMC::AccessPublic, QMetaType::QReal, {{
            { QMetaType::Int, 10 },
        }}),
        // Method 'setValue'
        QtMocHelpers::MethodData<void(int, qreal)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::QReal, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'values'
        QtMocHelpers::PropertyData<QVariantList>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'count'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'brushFilename'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2, 0x104),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ValuePositions'
        QtMocHelpers::EnumData<enum ValuePositions>(23, 23, QMC::EnumFlags{}).add({
            {   24, ValuePositions::LowerExtreme },
            {   25, ValuePositions::LowerQuartile },
            {   26, ValuePositions::Median },
            {   27, ValuePositions::UpperQuartile },
            {   28, ValuePositions::UpperExtreme },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeBoxSet, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeBoxSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QBoxSet::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DeclarativeBoxSetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DeclarativeBoxSetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17DeclarativeBoxSetE_t>.metaTypes,
    nullptr
} };

void DeclarativeBoxSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeBoxSet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changedValues(); break;
        case 1: _t->changedValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->brushFilenameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->handleBrushChanged(); break;
        case 4: _t->append((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 5: _t->clear(); break;
        case 6: { qreal _r = _t->at((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxSet::*)()>(_a, &DeclarativeBoxSet::changedValues, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxSet::*)(int )>(_a, &DeclarativeBoxSet::changedValue, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxSet::*)(const QString & )>(_a, &DeclarativeBoxSet::brushFilenameChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->values(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->brushFilename(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setValues(*reinterpret_cast<QVariantList*>(_v)); break;
        case 1: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setBrushFilename(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeBoxSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeBoxSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DeclarativeBoxSetE_t>.strings))
        return static_cast<void*>(this);
    return QBoxSet::qt_metacast(_clname);
}

int DeclarativeBoxSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QBoxSet::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeBoxSet::changedValues()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DeclarativeBoxSet::changedValue(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeBoxSet::brushFilenameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeBoxPlotSeries::qt_create_metaobjectdata<qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeBoxPlotSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "BoxPlotSeries",
        "QML.AddedInVersion",
        "259",
        "QML.ExtraVersion",
        "512",
        "axisXChanged",
        "",
        "QAbstractAxis*",
        "axis",
        "axisYChanged",
        "axisXTopChanged",
        "axisYRightChanged",
        "clicked",
        "DeclarativeBoxSet*",
        "boxset",
        "hovered",
        "status",
        "pressed",
        "released",
        "doubleClicked",
        "brushFilenameChanged",
        "brushFilename",
        "onHovered",
        "QBoxSet*",
        "onClicked",
        "onPressed",
        "onReleased",
        "onDoubleClicked",
        "handleBrushChanged",
        "at",
        "index",
        "append",
        "label",
        "QVariantList",
        "values",
        "box",
        "insert",
        "remove",
        "clear",
        "axisX",
        "axisY",
        "axisXTop",
        "axisYRight",
        "QQmlListProperty<QObject>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(13, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'axisXTopChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(14, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'axisYRightChanged'
        QtMocHelpers::SignalData<void(QAbstractAxis *)>(15, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(DeclarativeBoxSet *)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool, DeclarativeBoxSet *)>(19, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(DeclarativeBoxSet *)>(21, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(DeclarativeBoxSet *)>(22, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(DeclarativeBoxSet *)>(23, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'brushFilenameChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(24, 10, QMC::AccessPublic, 0x104, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'onHovered'
        QtMocHelpers::SlotData<void(bool, QBoxSet *)>(26, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 }, { 0x80000000 | 27, 18 },
        }}),
        // Slot 'onClicked'
        QtMocHelpers::SlotData<void(QBoxSet *)>(28, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 18 },
        }}),
        // Slot 'onPressed'
        QtMocHelpers::SlotData<void(QBoxSet *)>(29, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 18 },
        }}),
        // Slot 'onReleased'
        QtMocHelpers::SlotData<void(QBoxSet *)>(30, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 18 },
        }}),
        // Slot 'onDoubleClicked'
        QtMocHelpers::SlotData<void(QBoxSet *)>(31, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 18 },
        }}),
        // Slot 'handleBrushChanged'
        QtMocHelpers::SlotData<void()>(32, 10, QMC::AccessPrivate, QMetaType::Void),
        // Method 'at'
        QtMocHelpers::MethodData<DeclarativeBoxSet *(int)>(33, 10, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::Int, 34 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<DeclarativeBoxSet *(const QString, QVariantList)>(35, 10, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 36 }, { 0x80000000 | 37, 38 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(DeclarativeBoxSet *)>(35, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 39 },
        }}),
        // Method 'insert'
        QtMocHelpers::MethodData<DeclarativeBoxSet *(int, const QString, QVariantList)>(40, 10, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::Int, 34 }, { QMetaType::QString, 36 }, { 0x80000000 | 37, 38 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(DeclarativeBoxSet *)>(41, 10, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 17, 39 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(42, 10, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'axisX'
        QtMocHelpers::PropertyData<QAbstractAxis*>(43, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'axisY'
        QtMocHelpers::PropertyData<QAbstractAxis*>(44, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'axisXTop'
        QtMocHelpers::PropertyData<QAbstractAxis*>(45, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'axisYRight'
        QtMocHelpers::PropertyData<QAbstractAxis*>(46, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'brushFilename'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9, 0x104),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeBoxPlotSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeBoxPlotSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QBoxPlotSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeBoxPlotSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeBoxPlotSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 1: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 2: _t->axisXTopChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 3: _t->axisYRightChanged((*reinterpret_cast<std::add_pointer_t<QAbstractAxis*>>(_a[1]))); break;
        case 4: _t->clicked((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1]))); break;
        case 5: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[2]))); break;
        case 6: _t->pressed((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1]))); break;
        case 7: _t->released((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1]))); break;
        case 8: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1]))); break;
        case 9: _t->brushFilenameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->onHovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBoxSet*>>(_a[2]))); break;
        case 11: _t->onClicked((*reinterpret_cast<std::add_pointer_t<QBoxSet*>>(_a[1]))); break;
        case 12: _t->onPressed((*reinterpret_cast<std::add_pointer_t<QBoxSet*>>(_a[1]))); break;
        case 13: _t->onReleased((*reinterpret_cast<std::add_pointer_t<QBoxSet*>>(_a[1]))); break;
        case 14: _t->onDoubleClicked((*reinterpret_cast<std::add_pointer_t<QBoxSet*>>(_a[1]))); break;
        case 15: _t->handleBrushChanged(); break;
        case 16: { DeclarativeBoxSet* _r = _t->at((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DeclarativeBoxSet**>(_a[0]) = std::move(_r); }  break;
        case 17: { DeclarativeBoxSet* _r = _t->append((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<DeclarativeBoxSet**>(_a[0]) = std::move(_r); }  break;
        case 18: _t->append((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1]))); break;
        case 19: { DeclarativeBoxSet* _r = _t->insert((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3])));
            if (_a[0]) *reinterpret_cast<DeclarativeBoxSet**>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<DeclarativeBoxSet*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractAxis* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBoxSet* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBoxSet* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBoxSet* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBoxSet* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBoxSet* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeclarativeBoxSet* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(QAbstractAxis * )>(_a, &DeclarativeBoxPlotSeries::axisXChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(QAbstractAxis * )>(_a, &DeclarativeBoxPlotSeries::axisYChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(QAbstractAxis * )>(_a, &DeclarativeBoxPlotSeries::axisXTopChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(QAbstractAxis * )>(_a, &DeclarativeBoxPlotSeries::axisYRightChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(DeclarativeBoxSet * )>(_a, &DeclarativeBoxPlotSeries::clicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(bool , DeclarativeBoxSet * )>(_a, &DeclarativeBoxPlotSeries::hovered, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(DeclarativeBoxSet * )>(_a, &DeclarativeBoxPlotSeries::pressed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(DeclarativeBoxSet * )>(_a, &DeclarativeBoxPlotSeries::released, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(DeclarativeBoxSet * )>(_a, &DeclarativeBoxPlotSeries::doubleClicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBoxPlotSeries::*)(const QString & )>(_a, &DeclarativeBoxPlotSeries::brushFilenameChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisX(); break;
        case 1: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisY(); break;
        case 2: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisXTop(); break;
        case 3: *reinterpret_cast<QAbstractAxis**>(_v) = _t->axisYRight(); break;
        case 4: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->brushFilename(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAxisX(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 1: _t->setAxisY(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 2: _t->setAxisXTop(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 3: _t->setAxisYRight(*reinterpret_cast<QAbstractAxis**>(_v)); break;
        case 5: _t->setBrushFilename(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeBoxPlotSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeBoxPlotSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeBoxPlotSeriesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QBoxPlotSeries::qt_metacast(_clname);
}

int DeclarativeBoxPlotSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QBoxPlotSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeBoxPlotSeries::axisXChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeBoxPlotSeries::axisYChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeBoxPlotSeries::axisXTopChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeclarativeBoxPlotSeries::axisYRightChanged(QAbstractAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DeclarativeBoxPlotSeries::clicked(DeclarativeBoxSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DeclarativeBoxPlotSeries::hovered(bool _t1, DeclarativeBoxSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void DeclarativeBoxPlotSeries::pressed(DeclarativeBoxSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DeclarativeBoxPlotSeries::released(DeclarativeBoxSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DeclarativeBoxPlotSeries::doubleClicked(DeclarativeBoxSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DeclarativeBoxPlotSeries::brushFilenameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
