/****************************************************************************
** Meta object code from reading C++ file 'qabstractaxis.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/axis/qabstractaxis.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractaxis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QAbstractAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractAxis::qt_create_metaobjectdata<qt_meta_tag_ZN13QAbstractAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractAxis",
        "visibleChanged",
        "",
        "visible",
        "linePenChanged",
        "QPen",
        "pen",
        "lineVisibleChanged",
        "labelsVisibleChanged",
        "labelsBrushChanged",
        "QBrush",
        "brush",
        "labelsFontChanged",
        "QFont",
        "labelsAngleChanged",
        "angle",
        "gridLinePenChanged",
        "gridVisibleChanged",
        "minorGridVisibleChanged",
        "minorGridLinePenChanged",
        "gridLineColorChanged",
        "QColor",
        "color",
        "minorGridLineColorChanged",
        "colorChanged",
        "labelsColorChanged",
        "titleTextChanged",
        "title",
        "titleBrushChanged",
        "titleVisibleChanged",
        "titleFontChanged",
        "font",
        "shadesVisibleChanged",
        "shadesColorChanged",
        "shadesBorderColorChanged",
        "shadesPenChanged",
        "shadesBrushChanged",
        "reverseChanged",
        "reverse",
        "labelsEditableChanged",
        "editable",
        "labelsTruncatedChanged",
        "labelsTruncated",
        "truncateLabelsChanged",
        "truncateLabels",
        "lineVisible",
        "linePen",
        "labelsVisible",
        "labelsBrush",
        "labelsAngle",
        "labelsFont",
        "labelsColor",
        "gridVisible",
        "gridLinePen",
        "minorGridVisible",
        "minorGridLinePen",
        "gridLineColor",
        "minorGridLineColor",
        "shadesVisible",
        "shadesColor",
        "shadesBorderColor",
        "shadesPen",
        "shadesBrush",
        "titleText",
        "titleBrush",
        "titleVisible",
        "titleFont",
        "orientation",
        "Qt::Orientation",
        "alignment",
        "Qt::Alignment"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'linePenChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'lineVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'labelsVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'labelsBrushChanged'
        QtMocHelpers::SignalData<void(const QBrush &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'labelsFontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 6 },
        }}),
        // Signal 'labelsAngleChanged'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Signal 'gridLinePenChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'gridVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'minorGridVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'minorGridLinePenChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'gridLineColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'minorGridLineColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'labelsColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'titleTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 27 },
        }}),
        // Signal 'titleBrushChanged'
        QtMocHelpers::SignalData<void(const QBrush &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'titleVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'titleFontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 31 },
        }}),
        // Signal 'shadesVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'shadesColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'shadesBorderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'shadesPenChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'shadesBrushChanged'
        QtMocHelpers::SignalData<void(const QBrush &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'reverseChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Signal 'labelsEditableChanged'
        QtMocHelpers::SignalData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 40 },
        }}),
        // Signal 'labelsTruncatedChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(41, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { QMetaType::Bool, 42 },
        }}),
        // Signal 'truncateLabelsChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(43, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'lineVisible'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'linePen'
        QtMocHelpers::PropertyData<QPen>(46, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(22, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 13),
        // property 'labelsVisible'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'labelsBrush'
        QtMocHelpers::PropertyData<QBrush>(48, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'labelsAngle'
        QtMocHelpers::PropertyData<int>(49, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'labelsFont'
        QtMocHelpers::PropertyData<QFont>(50, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'labelsColor'
        QtMocHelpers::PropertyData<QColor>(51, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'labelsTruncated'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags, 26, 0x602),
        // property 'truncateLabels'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27, 0x602),
        // property 'gridVisible'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 8),
        // property 'gridLinePen'
        QtMocHelpers::PropertyData<QPen>(53, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'minorGridVisible'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 9),
        // property 'minorGridLinePen'
        QtMocHelpers::PropertyData<QPen>(55, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'gridLineColor'
        QtMocHelpers::PropertyData<QColor>(56, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'minorGridLineColor'
        QtMocHelpers::PropertyData<QColor>(57, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'shadesVisible'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'shadesColor'
        QtMocHelpers::PropertyData<QColor>(59, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 20),
        // property 'shadesBorderColor'
        QtMocHelpers::PropertyData<QColor>(60, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 21),
        // property 'shadesPen'
        QtMocHelpers::PropertyData<QPen>(61, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 22),
        // property 'shadesBrush'
        QtMocHelpers::PropertyData<QBrush>(62, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 23),
        // property 'titleText'
        QtMocHelpers::PropertyData<QString>(63, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'titleBrush'
        QtMocHelpers::PropertyData<QBrush>(64, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16),
        // property 'titleVisible'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'titleFont'
        QtMocHelpers::PropertyData<QFont>(66, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 18),
        // property 'orientation'
        QtMocHelpers::PropertyData<Qt::Orientation>(67, 0x80000000 | 68, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'alignment'
        QtMocHelpers::PropertyData<Qt::Alignment>(69, 0x80000000 | 70, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'reverse'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QAbstractAxis, qt_meta_tag_ZN13QAbstractAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstractAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QAbstractAxisE_t>.metaTypes,
    nullptr
} };

void QAbstractAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->visibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->linePenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 2: _t->lineVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->labelsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->labelsBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 5: _t->labelsFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 6: _t->labelsAngleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->gridLinePenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 8: _t->gridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->minorGridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->minorGridLinePenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 11: _t->gridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->minorGridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->labelsColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->titleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->titleBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 17: _t->titleVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->titleFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 19: _t->shadesVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->shadesColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 21: _t->shadesBorderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 22: _t->shadesPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 23: _t->shadesBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 24: _t->reverseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->labelsEditableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->labelsTruncatedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->truncateLabelsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::visibleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QPen & )>(_a, &QAbstractAxis::linePenChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::lineVisibleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::labelsVisibleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QBrush & )>(_a, &QAbstractAxis::labelsBrushChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QFont & )>(_a, &QAbstractAxis::labelsFontChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(int )>(_a, &QAbstractAxis::labelsAngleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QPen & )>(_a, &QAbstractAxis::gridLinePenChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::gridVisibleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::minorGridVisibleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QPen & )>(_a, &QAbstractAxis::minorGridLinePenChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QColor & )>(_a, &QAbstractAxis::gridLineColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QColor & )>(_a, &QAbstractAxis::minorGridLineColorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::colorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::labelsColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QString & )>(_a, &QAbstractAxis::titleTextChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QBrush & )>(_a, &QAbstractAxis::titleBrushChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::titleVisibleChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QFont & )>(_a, &QAbstractAxis::titleFontChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::shadesVisibleChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::shadesColorChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(QColor )>(_a, &QAbstractAxis::shadesBorderColorChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QPen & )>(_a, &QAbstractAxis::shadesPenChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(const QBrush & )>(_a, &QAbstractAxis::shadesBrushChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::reverseChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::labelsEditableChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::labelsTruncatedChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractAxis::*)(bool )>(_a, &QAbstractAxis::truncateLabelsChanged, 27))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isLineVisible(); break;
        case 2: *reinterpret_cast<QPen*>(_v) = _t->linePen(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->linePenColor(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->labelsVisible(); break;
        case 5: *reinterpret_cast<QBrush*>(_v) = _t->labelsBrush(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->labelsAngle(); break;
        case 7: *reinterpret_cast<QFont*>(_v) = _t->labelsFont(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->labelsColor(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->labelsTruncated(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->truncateLabels(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isGridLineVisible(); break;
        case 12: *reinterpret_cast<QPen*>(_v) = _t->gridLinePen(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isMinorGridLineVisible(); break;
        case 14: *reinterpret_cast<QPen*>(_v) = _t->minorGridLinePen(); break;
        case 15: *reinterpret_cast<QColor*>(_v) = _t->gridLineColor(); break;
        case 16: *reinterpret_cast<QColor*>(_v) = _t->minorGridLineColor(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->shadesVisible(); break;
        case 18: *reinterpret_cast<QColor*>(_v) = _t->shadesColor(); break;
        case 19: *reinterpret_cast<QColor*>(_v) = _t->shadesBorderColor(); break;
        case 20: *reinterpret_cast<QPen*>(_v) = _t->shadesPen(); break;
        case 21: *reinterpret_cast<QBrush*>(_v) = _t->shadesBrush(); break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->titleText(); break;
        case 23: *reinterpret_cast<QBrush*>(_v) = _t->titleBrush(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isTitleVisible(); break;
        case 25: *reinterpret_cast<QFont*>(_v) = _t->titleFont(); break;
        case 26: *reinterpret_cast<Qt::Orientation*>(_v) = _t->orientation(); break;
        case 27: *reinterpret_cast<Qt::Alignment*>(_v) = _t->alignment(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->isReverse(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setLineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLinePen(*reinterpret_cast<QPen*>(_v)); break;
        case 3: _t->setLinePenColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setLabelsBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 6: _t->setLabelsAngle(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setLabelsFont(*reinterpret_cast<QFont*>(_v)); break;
        case 8: _t->setLabelsColor(*reinterpret_cast<QColor*>(_v)); break;
        case 10: _t->setTruncateLabels(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setGridLineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setGridLinePen(*reinterpret_cast<QPen*>(_v)); break;
        case 13: _t->setMinorGridLineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setMinorGridLinePen(*reinterpret_cast<QPen*>(_v)); break;
        case 15: _t->setGridLineColor(*reinterpret_cast<QColor*>(_v)); break;
        case 16: _t->setMinorGridLineColor(*reinterpret_cast<QColor*>(_v)); break;
        case 17: _t->setShadesVisible(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setShadesColor(*reinterpret_cast<QColor*>(_v)); break;
        case 19: _t->setShadesBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 20: _t->setShadesPen(*reinterpret_cast<QPen*>(_v)); break;
        case 21: _t->setShadesBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 22: _t->setTitleText(*reinterpret_cast<QString*>(_v)); break;
        case 23: _t->setTitleBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 24: _t->setTitleVisible(*reinterpret_cast<bool*>(_v)); break;
        case 25: _t->setTitleFont(*reinterpret_cast<QFont*>(_v)); break;
        case 28: _t->setReverse(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QAbstractAxisE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstractAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void QAbstractAxis::visibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstractAxis::linePenChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstractAxis::lineVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstractAxis::labelsVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstractAxis::labelsBrushChanged(const QBrush & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstractAxis::labelsFontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstractAxis::labelsAngleChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstractAxis::gridLinePenChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstractAxis::gridVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstractAxis::minorGridVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstractAxis::minorGridLinePenChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstractAxis::gridLineColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstractAxis::minorGridLineColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstractAxis::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstractAxis::labelsColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QAbstractAxis::titleTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QAbstractAxis::titleBrushChanged(const QBrush & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QAbstractAxis::titleVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QAbstractAxis::titleFontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QAbstractAxis::shadesVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QAbstractAxis::shadesColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QAbstractAxis::shadesBorderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void QAbstractAxis::shadesPenChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void QAbstractAxis::shadesBrushChanged(const QBrush & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void QAbstractAxis::reverseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void QAbstractAxis::labelsEditableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void QAbstractAxis::labelsTruncatedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void QAbstractAxis::truncateLabelsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}
QT_WARNING_POP
