/****************************************************************************
** Meta object code from reading C++ file 'qcandlestickseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/candlestickchart/qcandlestickseries_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcandlestickseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto QCandlestickSeriesPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCandlestickSeriesPrivate",
        "clicked",
        "",
        "index",
        "QCandlestickSet*",
        "set",
        "pressed",
        "released",
        "doubleClicked",
        "updated",
        "updatedLayout",
        "updatedCandlesticks",
        "handleSeriesChange",
        "QAbstractSeries*",
        "series",
        "handleSeriesRemove"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(int, QCandlestickSet *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(int, QCandlestickSet *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(int, QCandlestickSet *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(int, QCandlestickSet *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'updated'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updatedLayout'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updatedCandlesticks'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSeriesChange'
        QtMocHelpers::SlotData<void(QAbstractSeries *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'handleSeriesRemove'
        QtMocHelpers::SlotData<void(QAbstractSeries *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QCandlestickSeriesPrivate, qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QCandlestickSeriesPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeriesPrivate::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>.metaTypes,
    nullptr
} };

void QCandlestickSeriesPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCandlestickSeriesPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 1: _t->pressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 2: _t->released((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 3: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 4: _t->updated(); break;
        case 5: _t->updatedLayout(); break;
        case 6: _t->updatedCandlesticks(); break;
        case 7: _t->handleSeriesChange((*reinterpret_cast<std::add_pointer_t<QAbstractSeries*>>(_a[1]))); break;
        case 8: _t->handleSeriesRemove((*reinterpret_cast<std::add_pointer_t<QAbstractSeries*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSeries* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)(int , QCandlestickSet * )>(_a, &QCandlestickSeriesPrivate::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)(int , QCandlestickSet * )>(_a, &QCandlestickSeriesPrivate::pressed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)(int , QCandlestickSet * )>(_a, &QCandlestickSeriesPrivate::released, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)(int , QCandlestickSet * )>(_a, &QCandlestickSeriesPrivate::doubleClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)()>(_a, &QCandlestickSeriesPrivate::updated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)()>(_a, &QCandlestickSeriesPrivate::updatedLayout, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeriesPrivate::*)()>(_a, &QCandlestickSeriesPrivate::updatedCandlesticks, 6))
            return;
    }
}

const QMetaObject *QCandlestickSeriesPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCandlestickSeriesPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QCandlestickSeriesPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeriesPrivate::qt_metacast(_clname);
}

int QCandlestickSeriesPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeriesPrivate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QCandlestickSeriesPrivate::clicked(int _t1, QCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QCandlestickSeriesPrivate::pressed(int _t1, QCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QCandlestickSeriesPrivate::released(int _t1, QCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QCandlestickSeriesPrivate::doubleClicked(int _t1, QCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QCandlestickSeriesPrivate::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QCandlestickSeriesPrivate::updatedLayout()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QCandlestickSeriesPrivate::updatedCandlesticks()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
