/****************************************************************************
** Meta object code from reading C++ file 'qcandlestickseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/candlestickchart/qcandlestickseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcandlestickseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QCandlestickSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QCandlestickSeries::qt_create_metaobjectdata<qt_meta_tag_ZN18QCandlestickSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCandlestickSeries",
        "clicked",
        "",
        "QCandlestickSet*",
        "set",
        "hovered",
        "status",
        "pressed",
        "released",
        "doubleClicked",
        "candlestickSetsAdded",
        "QList<QCandlestickSet*>",
        "sets",
        "candlestickSetsRemoved",
        "countChanged",
        "maximumColumnWidthChanged",
        "minimumColumnWidthChanged",
        "bodyWidthChanged",
        "bodyOutlineVisibilityChanged",
        "capsWidthChanged",
        "capsVisibilityChanged",
        "increasingColorChanged",
        "decreasingColorChanged",
        "brushChanged",
        "penChanged",
        "count",
        "maximumColumnWidth",
        "minimumColumnWidth",
        "bodyWidth",
        "bodyOutlineVisible",
        "capsWidth",
        "capsVisible",
        "increasingColor",
        "QColor",
        "decreasingColor",
        "brush",
        "QBrush",
        "pen",
        "QPen"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(QCandlestickSet *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool, QCandlestickSet *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(QCandlestickSet *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(QCandlestickSet *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(QCandlestickSet *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'candlestickSetsAdded'
        QtMocHelpers::SignalData<void(const QList<QCandlestickSet*> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'candlestickSetsRemoved'
        QtMocHelpers::SignalData<void(const QList<QCandlestickSet*> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximumColumnWidthChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumColumnWidthChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bodyWidthChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bodyOutlineVisibilityChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capsWidthChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capsVisibilityChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'increasingColorChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'decreasingColorChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'brushChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'penChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags, 7),
        // property 'maximumColumnWidth'
        QtMocHelpers::PropertyData<qreal>(26, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'minimumColumnWidth'
        QtMocHelpers::PropertyData<qreal>(27, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'bodyWidth'
        QtMocHelpers::PropertyData<qreal>(28, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'bodyOutlineVisible'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'capsWidth'
        QtMocHelpers::PropertyData<qreal>(30, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'capsVisible'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'increasingColor'
        QtMocHelpers::PropertyData<QColor>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'decreasingColor'
        QtMocHelpers::PropertyData<QColor>(34, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
        // property 'brush'
        QtMocHelpers::PropertyData<QBrush>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16),
        // property 'pen'
        QtMocHelpers::PropertyData<QPen>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QCandlestickSeries, qt_meta_tag_ZN18QCandlestickSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QCandlestickSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QCandlestickSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QCandlestickSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QCandlestickSeriesE_t>.metaTypes,
    nullptr
} };

void QCandlestickSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCandlestickSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[2]))); break;
        case 2: _t->pressed((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 3: _t->released((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QCandlestickSet*>>(_a[1]))); break;
        case 5: _t->candlestickSetsAdded((*reinterpret_cast<std::add_pointer_t<QList<QCandlestickSet*>>>(_a[1]))); break;
        case 6: _t->candlestickSetsRemoved((*reinterpret_cast<std::add_pointer_t<QList<QCandlestickSet*>>>(_a[1]))); break;
        case 7: _t->countChanged(); break;
        case 8: _t->maximumColumnWidthChanged(); break;
        case 9: _t->minimumColumnWidthChanged(); break;
        case 10: _t->bodyWidthChanged(); break;
        case 11: _t->bodyOutlineVisibilityChanged(); break;
        case 12: _t->capsWidthChanged(); break;
        case 13: _t->capsVisibilityChanged(); break;
        case 14: _t->increasingColorChanged(); break;
        case 15: _t->decreasingColorChanged(); break;
        case 16: _t->brushChanged(); break;
        case 17: _t->penChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(QCandlestickSet * )>(_a, &QCandlestickSeries::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(bool , QCandlestickSet * )>(_a, &QCandlestickSeries::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(QCandlestickSet * )>(_a, &QCandlestickSeries::pressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(QCandlestickSet * )>(_a, &QCandlestickSeries::released, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(QCandlestickSet * )>(_a, &QCandlestickSeries::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(const QList<QCandlestickSet*> & )>(_a, &QCandlestickSeries::candlestickSetsAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)(const QList<QCandlestickSet*> & )>(_a, &QCandlestickSeries::candlestickSetsRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::countChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::maximumColumnWidthChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::minimumColumnWidthChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::bodyWidthChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::bodyOutlineVisibilityChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::capsWidthChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::capsVisibilityChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::increasingColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::decreasingColorChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::brushChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCandlestickSeries::*)()>(_a, &QCandlestickSeries::penChanged, 17))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->maximumColumnWidth(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->minimumColumnWidth(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->bodyWidth(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->bodyOutlineVisible(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->capsWidth(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->capsVisible(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->increasingColor(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->decreasingColor(); break;
        case 9: *reinterpret_cast<QBrush*>(_v) = _t->brush(); break;
        case 10: *reinterpret_cast<QPen*>(_v) = _t->pen(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setMaximumColumnWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setMinimumColumnWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setBodyWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setBodyOutlineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setCapsWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setCapsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setIncreasingColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setDecreasingColor(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 10: _t->setPen(*reinterpret_cast<QPen*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QCandlestickSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCandlestickSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QCandlestickSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QCandlestickSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void QCandlestickSeries::clicked(QCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QCandlestickSeries::hovered(bool _t1, QCandlestickSet * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QCandlestickSeries::pressed(QCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QCandlestickSeries::released(QCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QCandlestickSeries::doubleClicked(QCandlestickSet * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QCandlestickSeries::candlestickSetsAdded(const QList<QCandlestickSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QCandlestickSeries::candlestickSetsRemoved(const QList<QCandlestickSet*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QCandlestickSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QCandlestickSeries::maximumColumnWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QCandlestickSeries::minimumColumnWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QCandlestickSeries::bodyWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QCandlestickSeries::bodyOutlineVisibilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QCandlestickSeries::capsWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QCandlestickSeries::capsVisibilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QCandlestickSeries::increasingColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QCandlestickSeries::decreasingColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QCandlestickSeries::brushChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QCandlestickSeries::penChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
