/****************************************************************************
** Meta object code from reading C++ file 'qhcandlestickmodelmapper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/candlestickchart/qhcandlestickmodelmapper.h"
#include <QtCharts/qcandlestickseries.h>
#include <QtCore/qabstractitemmodel.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qhcandlestickmodelmapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24QHCandlestickModelMapperE_t {};
} // unnamed namespace

template <> constexpr inline auto QHCandlestickModelMapper::qt_create_metaobjectdata<qt_meta_tag_ZN24QHCandlestickModelMapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHCandlestickModelMapper",
        "timestampColumnChanged",
        "",
        "openColumnChanged",
        "highColumnChanged",
        "lowColumnChanged",
        "closeColumnChanged",
        "firstSetRowChanged",
        "lastSetRowChanged",
        "timestampColumn",
        "openColumn",
        "highColumn",
        "lowColumn",
        "closeColumn",
        "firstSetRow",
        "lastSetRow"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'timestampColumnChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openColumnChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'highColumnChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lowColumnChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeColumnChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firstSetRowChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastSetRowChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'timestampColumn'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'openColumn'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'highColumn'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'lowColumn'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'closeColumn'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'firstSetRow'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'lastSetRow'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QHCandlestickModelMapper, qt_meta_tag_ZN24QHCandlestickModelMapperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHCandlestickModelMapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QCandlestickModelMapper::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QHCandlestickModelMapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QHCandlestickModelMapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QHCandlestickModelMapperE_t>.metaTypes,
    nullptr
} };

void QHCandlestickModelMapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHCandlestickModelMapper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->timestampColumnChanged(); break;
        case 1: _t->openColumnChanged(); break;
        case 2: _t->highColumnChanged(); break;
        case 3: _t->lowColumnChanged(); break;
        case 4: _t->closeColumnChanged(); break;
        case 5: _t->firstSetRowChanged(); break;
        case 6: _t->lastSetRowChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::timestampColumnChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::openColumnChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::highColumnChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::lowColumnChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::closeColumnChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::firstSetRowChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHCandlestickModelMapper::*)()>(_a, &QHCandlestickModelMapper::lastSetRowChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->timestampColumn(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->openColumn(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->highColumn(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->lowColumn(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->closeColumn(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->firstSetRow(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->lastSetRow(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTimestampColumn(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setOpenColumn(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setHighColumn(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setLowColumn(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setCloseColumn(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setFirstSetRow(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setLastSetRow(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QHCandlestickModelMapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHCandlestickModelMapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QHCandlestickModelMapperE_t>.strings))
        return static_cast<void*>(this);
    return QCandlestickModelMapper::qt_metacast(_clname);
}

int QHCandlestickModelMapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCandlestickModelMapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QHCandlestickModelMapper::timestampColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QHCandlestickModelMapper::openColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QHCandlestickModelMapper::highColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QHCandlestickModelMapper::lowColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QHCandlestickModelMapper::closeColumnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QHCandlestickModelMapper::firstSetRowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QHCandlestickModelMapper::lastSetRowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
