/****************************************************************************
** Meta object code from reading C++ file 'qareaseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/areachart/qareaseries.h"
#include <QtCharts/qlineseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qareaseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QAreaSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QAreaSeries::qt_create_metaobjectdata<qt_meta_tag_ZN11QAreaSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAreaSeries",
        "clicked",
        "",
        "QPointF",
        "point",
        "hovered",
        "state",
        "pressed",
        "released",
        "doubleClicked",
        "selected",
        "colorChanged",
        "QColor",
        "color",
        "borderColorChanged",
        "pointLabelsFormatChanged",
        "format",
        "pointLabelsVisibilityChanged",
        "visible",
        "pointLabelsFontChanged",
        "QFont",
        "font",
        "pointLabelsColorChanged",
        "pointLabelsClippingChanged",
        "clipping",
        "upperSeries",
        "QLineSeries*",
        "lowerSeries",
        "borderColor",
        "pointLabelsFormat",
        "pointLabelsVisible",
        "pointLabelsFont",
        "pointLabelsColor",
        "pointLabelsClipping"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(const QPointF &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(const QPointF &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(const QPointF &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(const QPointF &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(const QPointF &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'selected'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'pointLabelsFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'pointLabelsVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'pointLabelsFontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'pointLabelsColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'pointLabelsClippingChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'upperSeries'
        QtMocHelpers::PropertyData<QLineSeries*>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'lowerSeries'
        QtMocHelpers::PropertyData<QLineSeries*>(27, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(28, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'pointLabelsFormat'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'pointLabelsVisible'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'pointLabelsFont'
        QtMocHelpers::PropertyData<QFont>(31, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'pointLabelsColor'
        QtMocHelpers::PropertyData<QColor>(32, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'pointLabelsClipping'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QAreaSeries, qt_meta_tag_ZN11QAreaSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAreaSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QAreaSeriesE_t>.metaTypes,
    nullptr
} };

void QAreaSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAreaSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 3: _t->released((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 5: _t->selected(); break;
        case 6: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->pointLabelsFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->pointLabelsVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->pointLabelsFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 11: _t->pointLabelsColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->pointLabelsClippingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QPointF & )>(_a, &QAreaSeries::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QPointF & , bool )>(_a, &QAreaSeries::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QPointF & )>(_a, &QAreaSeries::pressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QPointF & )>(_a, &QAreaSeries::released, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QPointF & )>(_a, &QAreaSeries::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)()>(_a, &QAreaSeries::selected, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::colorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(QColor )>(_a, &QAreaSeries::borderColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QString & )>(_a, &QAreaSeries::pointLabelsFormatChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(bool )>(_a, &QAreaSeries::pointLabelsVisibilityChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QFont & )>(_a, &QAreaSeries::pointLabelsFontChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(const QColor & )>(_a, &QAreaSeries::pointLabelsColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAreaSeries::*)(bool )>(_a, &QAreaSeries::pointLabelsClippingChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QLineSeries**>(_v) = _t->upperSeries(); break;
        case 1: *reinterpret_cast<QLineSeries**>(_v) = _t->lowerSeries(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->pointLabelsFormat(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->pointLabelsVisible(); break;
        case 6: *reinterpret_cast<QFont*>(_v) = _t->pointLabelsFont(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->pointLabelsColor(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->pointLabelsClipping(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setPointLabelsFormat(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setPointLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setPointLabelsFont(*reinterpret_cast<QFont*>(_v)); break;
        case 7: _t->setPointLabelsColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setPointLabelsClipping(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAreaSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAreaSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QAreaSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QAreaSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QAreaSeries::clicked(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAreaSeries::hovered(const QPointF & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QAreaSeries::pressed(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAreaSeries::released(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAreaSeries::doubleClicked(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAreaSeries::selected()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QAreaSeries::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAreaSeries::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAreaSeries::pointLabelsFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAreaSeries::pointLabelsVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAreaSeries::pointLabelsFontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAreaSeries::pointLabelsColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAreaSeries::pointLabelsClippingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
