/****************************************************************************
** Meta object code from reading C++ file 'qlegend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/legend/qlegend.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlegend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QLegendE_t {};
} // unnamed namespace

template <> constexpr inline auto QLegend::qt_create_metaobjectdata<qt_meta_tag_ZN7QLegendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLegend",
        "backgroundVisibleChanged",
        "",
        "visible",
        "colorChanged",
        "QColor",
        "color",
        "borderColorChanged",
        "fontChanged",
        "QFont",
        "font",
        "labelColorChanged",
        "reverseMarkersChanged",
        "reverseMarkers",
        "showToolTipsChanged",
        "showToolTips",
        "markerShapeChanged",
        "MarkerShape",
        "shape",
        "attachedToChartChanged",
        "attachedToChart",
        "interactiveChanged",
        "interactive",
        "alignment",
        "Qt::Alignment",
        "backgroundVisible",
        "borderColor",
        "labelColor",
        "markerShape",
        "MarkerShapeDefault",
        "MarkerShapeRectangle",
        "MarkerShapeCircle",
        "MarkerShapeFromSeries",
        "MarkerShapeRotatedRectangle",
        "MarkerShapeTriangle",
        "MarkerShapeStar",
        "MarkerShapePentagon"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'backgroundVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(QFont)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'labelColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'reverseMarkersChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'showToolTipsChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'markerShapeChanged'
        QtMocHelpers::SignalData<void(enum MarkerShape)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'attachedToChartChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(19, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'interactiveChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'alignment'
        QtMocHelpers::PropertyData<Qt::Alignment>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'backgroundVisible'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(26, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'font'
        QtMocHelpers::PropertyData<QFont>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'labelColor'
        QtMocHelpers::PropertyData<QColor>(27, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'reverseMarkers'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'showToolTips'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'markerShape'
        QtMocHelpers::PropertyData<enum MarkerShape>(28, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MarkerShape'
        QtMocHelpers::EnumData<enum MarkerShape>(17, 17, QMC::EnumFlags{}).add({
            {   29, MarkerShape::MarkerShapeDefault },
            {   30, MarkerShape::MarkerShapeRectangle },
            {   31, MarkerShape::MarkerShapeCircle },
            {   32, MarkerShape::MarkerShapeFromSeries },
            {   33, MarkerShape::MarkerShapeRotatedRectangle },
            {   34, MarkerShape::MarkerShapeTriangle },
            {   35, MarkerShape::MarkerShapeStar },
            {   36, MarkerShape::MarkerShapePentagon },
        }),
    };
    return QtMocHelpers::metaObjectData<QLegend, qt_meta_tag_ZN7QLegendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLegend::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QLegendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QLegendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QLegendE_t>.metaTypes,
    nullptr
} };

void QLegend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLegend *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->backgroundVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 4: _t->labelColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->reverseMarkersChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showToolTipsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->markerShapeChanged((*reinterpret_cast<std::add_pointer_t<enum MarkerShape>>(_a[1]))); break;
        case 8: _t->attachedToChartChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->interactiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(bool )>(_a, &QLegend::backgroundVisibleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(QColor )>(_a, &QLegend::colorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(QColor )>(_a, &QLegend::borderColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(QFont )>(_a, &QLegend::fontChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(QColor )>(_a, &QLegend::labelColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(bool )>(_a, &QLegend::reverseMarkersChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(bool )>(_a, &QLegend::showToolTipsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(MarkerShape )>(_a, &QLegend::markerShapeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(bool )>(_a, &QLegend::attachedToChartChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLegend::*)(bool )>(_a, &QLegend::interactiveChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Qt::Alignment*>(_v) = _t->alignment(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isBackgroundVisible(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 4: *reinterpret_cast<QFont*>(_v) = _t->font(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->labelColor(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->reverseMarkers(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->showToolTips(); break;
        case 8: *reinterpret_cast<enum MarkerShape*>(_v) = _t->markerShape(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAlignment(*reinterpret_cast<Qt::Alignment*>(_v)); break;
        case 1: _t->setBackgroundVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setFont(*reinterpret_cast<QFont*>(_v)); break;
        case 5: _t->setLabelColor(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setReverseMarkers(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setShowToolTips(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setMarkerShape(*reinterpret_cast<enum MarkerShape*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QLegend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLegend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QLegendE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsWidget::qt_metacast(_clname);
}

int QLegend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QLegend::backgroundVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QLegend::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QLegend::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QLegend::fontChanged(QFont _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QLegend::labelColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QLegend::reverseMarkersChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QLegend::showToolTipsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QLegend::markerShapeChanged(MarkerShape _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QLegend::attachedToChartChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QLegend::interactiveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
