/****************************************************************************
** Meta object code from reading C++ file 'qxyseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/xychart/qxyseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qxyseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QXYSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QXYSeries::qt_create_metaobjectdata<qt_meta_tag_ZN9QXYSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QXYSeries",
        "clicked",
        "",
        "QPointF",
        "point",
        "hovered",
        "state",
        "pressed",
        "released",
        "doubleClicked",
        "pointReplaced",
        "index",
        "pointRemoved",
        "pointAdded",
        "colorChanged",
        "QColor",
        "color",
        "selectedColorChanged",
        "pointsReplaced",
        "pointLabelsFormatChanged",
        "format",
        "pointLabelsVisibilityChanged",
        "visible",
        "pointLabelsFontChanged",
        "QFont",
        "font",
        "pointLabelsColorChanged",
        "pointLabelsClippingChanged",
        "clipping",
        "pointsRemoved",
        "count",
        "penChanged",
        "QPen",
        "pen",
        "selectedPointsChanged",
        "lightMarkerChanged",
        "QImage",
        "lightMarker",
        "selectedLightMarkerChanged",
        "selectedLightMarker",
        "bestFitLineVisibilityChanged",
        "bestFitLinePenChanged",
        "bestFitLineColorChanged",
        "pointsConfigurationChanged",
        "QXYSeries::PointsConfigurationHash",
        "configuration",
        "markerSizeChanged",
        "size",
        "pointsVisible",
        "selectedColor",
        "pointLabelsFormat",
        "pointLabelsVisible",
        "pointLabelsFont",
        "pointLabelsColor",
        "pointLabelsClipping",
        "bestFitLineVisible",
        "bestFitLineColor",
        "PointConfiguration",
        "Color",
        "Size",
        "Visibility",
        "LabelVisibility",
        "LabelFormat"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(const QPointF &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(const QPointF &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(const QPointF &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(const QPointF &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(const QPointF &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'pointReplaced'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'pointRemoved'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'pointAdded'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'selectedColorChanged'
        QtMocHelpers::RevisionedSignalData<void(const QColor &)>(17, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'pointsReplaced'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointLabelsFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'pointLabelsVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Signal 'pointLabelsFontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'pointLabelsColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'pointLabelsClippingChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Signal 'pointsRemoved'
        QtMocHelpers::SignalData<void(int, int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 30 },
        }}),
        // Signal 'penChanged'
        QtMocHelpers::SignalData<void(const QPen &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Signal 'selectedPointsChanged'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lightMarkerChanged'
        QtMocHelpers::RevisionedSignalData<void(const QImage &)>(35, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'selectedLightMarkerChanged'
        QtMocHelpers::RevisionedSignalData<void(const QImage &)>(38, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 36, 39 },
        }}),
        // Signal 'bestFitLineVisibilityChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(40, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Signal 'bestFitLinePenChanged'
        QtMocHelpers::RevisionedSignalData<void(const QPen &)>(41, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Signal 'bestFitLineColorChanged'
        QtMocHelpers::RevisionedSignalData<void(const QColor &)>(42, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'pointsConfigurationChanged'
        QtMocHelpers::RevisionedSignalData<void(const QXYSeries::PointsConfigurationHash &)>(43, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Signal 'markerSizeChanged'
        QtMocHelpers::SignalData<void(qreal)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'pointsVisible'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(16, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'selectedColor'
        QtMocHelpers::PropertyData<QColor>(49, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9, 0x602),
        // property 'pointLabelsFormat'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'pointLabelsVisible'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'pointLabelsFont'
        QtMocHelpers::PropertyData<QFont>(52, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13),
        // property 'pointLabelsColor'
        QtMocHelpers::PropertyData<QColor>(53, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'pointLabelsClipping'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'bestFitLineVisible'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21, 0x602),
        // property 'bestFitLineColor'
        QtMocHelpers::PropertyData<QColor>(56, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 23, 0x602),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PointConfiguration'
        QtMocHelpers::EnumData<enum PointConfiguration>(57, 57, QMC::EnumIsScoped).add({
            {   58, PointConfiguration::Color },
            {   59, PointConfiguration::Size },
            {   60, PointConfiguration::Visibility },
            {   61, PointConfiguration::LabelVisibility },
            {   62, PointConfiguration::LabelFormat },
        }),
    };
    return QtMocHelpers::metaObjectData<QXYSeries, qt_meta_tag_ZN9QXYSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QXYSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QXYSeriesE_t>.metaTypes,
    nullptr
} };

void QXYSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QXYSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 3: _t->released((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 5: _t->pointReplaced((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->pointRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->pointAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->selectedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 10: _t->pointsReplaced(); break;
        case 11: _t->pointLabelsFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->pointLabelsVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->pointLabelsFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 14: _t->pointLabelsColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->pointLabelsClippingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->pointsRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 17: _t->penChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 18: _t->selectedPointsChanged(); break;
        case 19: _t->lightMarkerChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 20: _t->selectedLightMarkerChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 21: _t->bestFitLineVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->bestFitLinePenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 23: _t->bestFitLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 24: _t->pointsConfigurationChanged((*reinterpret_cast<std::add_pointer_t<QXYSeries::PointsConfigurationHash>>(_a[1]))); break;
        case 25: _t->markerSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPointF & )>(_a, &QXYSeries::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPointF & , bool )>(_a, &QXYSeries::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPointF & )>(_a, &QXYSeries::pressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPointF & )>(_a, &QXYSeries::released, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPointF & )>(_a, &QXYSeries::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(int )>(_a, &QXYSeries::pointReplaced, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(int )>(_a, &QXYSeries::pointRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(int )>(_a, &QXYSeries::pointAdded, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(QColor )>(_a, &QXYSeries::colorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QColor & )>(_a, &QXYSeries::selectedColorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::pointsReplaced, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QString & )>(_a, &QXYSeries::pointLabelsFormatChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(bool )>(_a, &QXYSeries::pointLabelsVisibilityChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QFont & )>(_a, &QXYSeries::pointLabelsFontChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QColor & )>(_a, &QXYSeries::pointLabelsColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(bool )>(_a, &QXYSeries::pointLabelsClippingChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(int , int )>(_a, &QXYSeries::pointsRemoved, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPen & )>(_a, &QXYSeries::penChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)()>(_a, &QXYSeries::selectedPointsChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QImage & )>(_a, &QXYSeries::lightMarkerChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QImage & )>(_a, &QXYSeries::selectedLightMarkerChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(bool )>(_a, &QXYSeries::bestFitLineVisibilityChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QPen & )>(_a, &QXYSeries::bestFitLinePenChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QColor & )>(_a, &QXYSeries::bestFitLineColorChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(const QXYSeries::PointsConfigurationHash & )>(_a, &QXYSeries::pointsConfigurationChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QXYSeries::*)(qreal )>(_a, &QXYSeries::markerSizeChanged, 25))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->pointsVisible(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->pointLabelsFormat(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->pointLabelsVisible(); break;
        case 5: *reinterpret_cast<QFont*>(_v) = _t->pointLabelsFont(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->pointLabelsColor(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->pointLabelsClipping(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->bestFitLineVisible(); break;
        case 9: *reinterpret_cast<QColor*>(_v) = _t->bestFitLineColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPointsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setSelectedColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setPointLabelsFormat(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setPointLabelsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setPointLabelsFont(*reinterpret_cast<QFont*>(_v)); break;
        case 6: _t->setPointLabelsColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setPointLabelsClipping(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setBestFitLineVisible(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setBestFitLineColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QXYSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QXYSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QXYSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QXYSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QXYSeries::clicked(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QXYSeries::hovered(const QPointF & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QXYSeries::pressed(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QXYSeries::released(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QXYSeries::doubleClicked(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QXYSeries::pointReplaced(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QXYSeries::pointRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QXYSeries::pointAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QXYSeries::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QXYSeries::selectedColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QXYSeries::pointsReplaced()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QXYSeries::pointLabelsFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QXYSeries::pointLabelsVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QXYSeries::pointLabelsFontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QXYSeries::pointLabelsColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QXYSeries::pointLabelsClippingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QXYSeries::pointsRemoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void QXYSeries::penChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QXYSeries::selectedPointsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QXYSeries::lightMarkerChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QXYSeries::selectedLightMarkerChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QXYSeries::bestFitLineVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void QXYSeries::bestFitLinePenChanged(const QPen & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void QXYSeries::bestFitLineColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void QXYSeries::pointsConfigurationChanged(const QXYSeries::PointsConfigurationHash & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void QXYSeries::markerSizeChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}
QT_WARNING_POP
