/****************************************************************************
** Meta object code from reading C++ file 'qscatterseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/scatterchart/qscatterseries.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qscatterseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14QScatterSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QScatterSeries::qt_create_metaobjectdata<qt_meta_tag_ZN14QScatterSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScatterSeries",
        "colorChanged",
        "",
        "QColor",
        "color",
        "borderColorChanged",
        "markerShapeChanged",
        "MarkerShape",
        "shape",
        "markerSizeChanged",
        "size",
        "borderColor",
        "markerShape",
        "markerSize",
        "brush",
        "QBrush",
        "MarkerShapeCircle",
        "MarkerShapeRectangle",
        "MarkerShapeRotatedRectangle",
        "MarkerShapeTriangle",
        "MarkerShapeStar",
        "MarkerShapePentagon"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'markerShapeChanged'
        QtMocHelpers::SignalData<void(enum MarkerShape)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'markerSizeChanged'
        QtMocHelpers::SignalData<void(qreal)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(11, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'markerShape'
        QtMocHelpers::PropertyData<enum MarkerShape>(12, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'markerSize'
        QtMocHelpers::PropertyData<qreal>(13, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'brush'
        QtMocHelpers::PropertyData<QBrush>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MarkerShape'
        QtMocHelpers::EnumData<enum MarkerShape>(7, 7, QMC::EnumFlags{}).add({
            {   16, MarkerShape::MarkerShapeCircle },
            {   17, MarkerShape::MarkerShapeRectangle },
            {   18, MarkerShape::MarkerShapeRotatedRectangle },
            {   19, MarkerShape::MarkerShapeTriangle },
            {   20, MarkerShape::MarkerShapeStar },
            {   21, MarkerShape::MarkerShapePentagon },
        }),
    };
    return QtMocHelpers::metaObjectData<QScatterSeries, qt_meta_tag_ZN14QScatterSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScatterSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QXYSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QScatterSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QScatterSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14QScatterSeriesE_t>.metaTypes,
    nullptr
} };

void QScatterSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScatterSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->markerShapeChanged((*reinterpret_cast<std::add_pointer_t<enum MarkerShape>>(_a[1]))); break;
        case 3: _t->markerSizeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QScatterSeries::*)(QColor )>(_a, &QScatterSeries::colorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatterSeries::*)(QColor )>(_a, &QScatterSeries::borderColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatterSeries::*)(MarkerShape )>(_a, &QScatterSeries::markerShapeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScatterSeries::*)(qreal )>(_a, &QScatterSeries::markerSizeChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 2: *reinterpret_cast<enum MarkerShape*>(_v) = _t->markerShape(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->markerSize(); break;
        case 4: *reinterpret_cast<QBrush*>(_v) = _t->brush(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 1: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 2: _t->setMarkerShape(*reinterpret_cast<enum MarkerShape*>(_v)); break;
        case 3: _t->setMarkerSize(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setBrush(*reinterpret_cast<QBrush*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QScatterSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScatterSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QScatterSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QXYSeries::qt_metacast(_clname);
}

int QScatterSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QXYSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QScatterSeries::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QScatterSeries::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QScatterSeries::markerShapeChanged(MarkerShape _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QScatterSeries::markerSizeChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
