/****************************************************************************
** Meta object code from reading C++ file 'qbarset.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/barchart/qbarset.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbarset.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QBarSetE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarSet::qt_create_metaobjectdata<qt_meta_tag_ZN7QBarSetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarSet",
        "clicked",
        "",
        "index",
        "hovered",
        "status",
        "pressed",
        "released",
        "doubleClicked",
        "penChanged",
        "brushChanged",
        "labelChanged",
        "labelBrushChanged",
        "labelFontChanged",
        "colorChanged",
        "QColor",
        "color",
        "borderColorChanged",
        "labelColorChanged",
        "selectedColorChanged",
        "valuesAdded",
        "count",
        "valuesRemoved",
        "valueChanged",
        "selectedBarsChanged",
        "QList<int>",
        "indexes",
        "label",
        "pen",
        "QPen",
        "brush",
        "QBrush",
        "labelBrush",
        "labelFont",
        "QFont",
        "borderColor",
        "labelColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(bool, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 }, { QMetaType::Int, 3 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'penChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'brushChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelBrushChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'labelFontChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(QColor)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'borderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'labelColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'selectedColorChanged'
        QtMocHelpers::RevisionedSignalData<void(const QColor &)>(19, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'valuesAdded'
        QtMocHelpers::SignalData<void(int, int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 21 },
        }}),
        // Signal 'valuesRemoved'
        QtMocHelpers::SignalData<void(int, int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 21 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'selectedBarsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QList<int> &)>(24, 2, QMC::AccessPublic, 0x602, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'label'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'pen'
        QtMocHelpers::PropertyData<QPen>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'brush'
        QtMocHelpers::PropertyData<QBrush>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'labelBrush'
        QtMocHelpers::PropertyData<QBrush>(32, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'labelFont'
        QtMocHelpers::PropertyData<QFont>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(16, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'borderColor'
        QtMocHelpers::PropertyData<QColor>(35, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'labelColor'
        QtMocHelpers::PropertyData<QColor>(36, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QBarSet, qt_meta_tag_ZN7QBarSetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBarSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QBarSetE_t>.metaTypes,
    nullptr
} };

void QBarSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBarSet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->hovered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->pressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->released((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->penChanged(); break;
        case 6: _t->brushChanged(); break;
        case 7: _t->labelChanged(); break;
        case 8: _t->labelBrushChanged(); break;
        case 9: _t->labelFontChanged(); break;
        case 10: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 11: _t->borderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->labelColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->selectedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->valuesAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 15: _t->valuesRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 16: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->selectedBarsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int )>(_a, &QBarSet::clicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(bool , int )>(_a, &QBarSet::hovered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int )>(_a, &QBarSet::pressed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int )>(_a, &QBarSet::released, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int )>(_a, &QBarSet::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::penChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::brushChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::labelChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::labelBrushChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)()>(_a, &QBarSet::labelFontChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::colorChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::borderColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(QColor )>(_a, &QBarSet::labelColorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(const QColor & )>(_a, &QBarSet::selectedColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int , int )>(_a, &QBarSet::valuesAdded, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int , int )>(_a, &QBarSet::valuesRemoved, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(int )>(_a, &QBarSet::valueChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarSet::*)(const QList<int> & )>(_a, &QBarSet::selectedBarsChanged, 17))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 1: *reinterpret_cast<QPen*>(_v) = _t->pen(); break;
        case 2: *reinterpret_cast<QBrush*>(_v) = _t->brush(); break;
        case 3: *reinterpret_cast<QBrush*>(_v) = _t->labelBrush(); break;
        case 4: *reinterpret_cast<QFont*>(_v) = _t->labelFont(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->borderColor(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->labelColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setPen(*reinterpret_cast<QPen*>(_v)); break;
        case 2: _t->setBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 3: _t->setLabelBrush(*reinterpret_cast<QBrush*>(_v)); break;
        case 4: _t->setLabelFont(*reinterpret_cast<QFont*>(_v)); break;
        case 5: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setBorderColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setLabelColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBarSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBarSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QBarSetE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBarSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QBarSet::clicked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QBarSet::hovered(bool _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QBarSet::pressed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBarSet::released(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QBarSet::doubleClicked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QBarSet::penChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QBarSet::brushChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QBarSet::labelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QBarSet::labelBrushChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QBarSet::labelFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QBarSet::colorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QBarSet::borderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QBarSet::labelColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QBarSet::selectedColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QBarSet::valuesAdded(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2);
}

// SIGNAL 15
void QBarSet::valuesRemoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void QBarSet::valueChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QBarSet::selectedBarsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}
QT_WARNING_POP
