/****************************************************************************
** Meta object code from reading C++ file 'chartaxiselement_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/axis/chartaxiselement_p.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'chartaxiselement_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ChartAxisElementE_t {};
} // unnamed namespace

template <> constexpr inline auto ChartAxisElement::qt_create_metaobjectdata<qt_meta_tag_ZN16ChartAxisElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ChartAxisElement",
        "clicked",
        "",
        "handleVisibleChanged",
        "visible",
        "handleArrowVisibleChanged",
        "handleGridVisibleChanged",
        "handleLabelsVisibleChanged",
        "handleShadesVisibleChanged",
        "handleLabelsAngleChanged",
        "angle",
        "handleShadesBrushChanged",
        "QBrush",
        "brush",
        "handleShadesPenChanged",
        "QPen",
        "pen",
        "handleArrowPenChanged",
        "handleGridPenChanged",
        "handleMinorArrowPenChanged",
        "handleMinorGridPenChanged",
        "handleMinorGridLineColorChanged",
        "QColor",
        "color",
        "handleGridLineColorChanged",
        "handleLabelsBrushChanged",
        "handleLabelsFontChanged",
        "QFont",
        "font",
        "handleTitleBrushChanged",
        "handleTitleFontChanged",
        "handleTitleTextChanged",
        "title",
        "handleTitleVisibleChanged",
        "handleRangeChanged",
        "min",
        "max",
        "handleReverseChanged",
        "reverse",
        "handleMinorArrowVisibleChanged",
        "handleMinorGridVisibleChanged",
        "handleLabelsPositionChanged",
        "handleTruncateLabelsChanged",
        "handleColorScaleSizeChanged",
        "handleColorScaleGradientChanged",
        "valueLabelEdited",
        "oldValue",
        "newValue",
        "dateTimeLabelEdited",
        "oldTime",
        "newTime"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleArrowVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleGridVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleLabelsVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleShadesVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleLabelsAngleChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'handleShadesBrushChanged'
        QtMocHelpers::SlotData<void(const QBrush &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'handleShadesPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleArrowPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleGridPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleMinorArrowPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleMinorGridPenChanged'
        QtMocHelpers::SlotData<void(const QPen &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleMinorGridLineColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'handleGridLineColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'handleLabelsBrushChanged'
        QtMocHelpers::SlotData<void(const QBrush &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'handleLabelsFontChanged'
        QtMocHelpers::SlotData<void(const QFont &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'handleTitleBrushChanged'
        QtMocHelpers::SlotData<void(const QBrush &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'handleTitleFontChanged'
        QtMocHelpers::SlotData<void(const QFont &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'handleTitleTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'handleTitleVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleRangeChanged'
        QtMocHelpers::SlotData<void(qreal, qreal)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 35 }, { QMetaType::QReal, 36 },
        }}),
        // Slot 'handleReverseChanged'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'handleMinorArrowVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleMinorGridVisibleChanged'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'handleLabelsPositionChanged'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleTruncateLabelsChanged'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleColorScaleSizeChanged'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleColorScaleGradientChanged'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'valueLabelEdited'
        QtMocHelpers::SlotData<void(qreal, qreal)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 46 }, { QMetaType::QReal, 47 },
        }}),
        // Slot 'dateTimeLabelEdited'
        QtMocHelpers::SlotData<void(const QDateTime &, const QDateTime &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 49 }, { QMetaType::QDateTime, 50 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ChartAxisElement, qt_meta_tag_ZN16ChartAxisElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ChartAxisElement::staticMetaObject = { {
    QMetaObject::SuperData::link<ChartElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ChartAxisElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ChartAxisElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ChartAxisElementE_t>.metaTypes,
    nullptr
} };

void ChartAxisElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ChartAxisElement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked(); break;
        case 1: _t->handleVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->handleArrowVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->handleGridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->handleLabelsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->handleShadesVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->handleLabelsAngleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->handleShadesBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 8: _t->handleShadesPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 9: _t->handleArrowPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 10: _t->handleGridPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 11: _t->handleMinorArrowPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 12: _t->handleMinorGridPenChanged((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 13: _t->handleMinorGridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->handleGridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->handleLabelsBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 16: _t->handleLabelsFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 17: _t->handleTitleBrushChanged((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 18: _t->handleTitleFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 19: _t->handleTitleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->handleTitleVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->handleRangeChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 22: _t->handleReverseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->handleMinorArrowVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->handleMinorGridVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->handleLabelsPositionChanged(); break;
        case 26: _t->handleTruncateLabelsChanged(); break;
        case 27: _t->handleColorScaleSizeChanged(); break;
        case 28: _t->handleColorScaleGradientChanged(); break;
        case 29: _t->valueLabelEdited((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 30: _t->dateTimeLabelEdited((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ChartAxisElement::*)()>(_a, &ChartAxisElement::clicked, 0))
            return;
    }
}

const QMetaObject *ChartAxisElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ChartAxisElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ChartAxisElementE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsLayoutItem"))
        return static_cast< QGraphicsLayoutItem*>(this);
    return ChartElement::qt_metacast(_clname);
}

int ChartAxisElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChartElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void ChartAxisElement::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
