/****************************************************************************
** Meta object code from reading C++ file 'declarativecategoryaxis_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/chartsqml2/declarativecategoryaxis_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativecategoryaxis_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24DeclarativeCategoryRangeE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeCategoryRange::qt_create_metaobjectdata<qt_meta_tag_ZN24DeclarativeCategoryRangeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeCategoryRange",
        "QML.Element",
        "CategoryRange",
        "QML.AddedInVersion",
        "257",
        "QML.ExtraVersion",
        "512",
        "endValue",
        "label"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'endValue'
        QtMocHelpers::PropertyData<qreal>(7, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeCategoryRange, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeCategoryRange::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeCategoryRangeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeCategoryRangeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24DeclarativeCategoryRangeE_t>.metaTypes,
    nullptr
} };

void DeclarativeCategoryRange::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeCategoryRange *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->endValue(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEndValue(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeCategoryRange::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeCategoryRange::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24DeclarativeCategoryRangeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DeclarativeCategoryRange::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23DeclarativeCategoryAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeCategoryAxis::qt_create_metaobjectdata<qt_meta_tag_ZN23DeclarativeCategoryAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeCategoryAxis",
        "DefaultProperty",
        "axisChildren",
        "QML.Element",
        "CategoryAxis",
        "QML.AddedInVersion",
        "257",
        "QML.ExtraVersion",
        "512",
        "labelsPositionChanged",
        "",
        "AxisLabelsPosition",
        "position",
        "append",
        "label",
        "categoryEndValue",
        "remove",
        "replace",
        "oldLabel",
        "newLabel",
        "QQmlListProperty<QObject>",
        "labelsPosition",
        "AxisLabelsPositionCenter",
        "AxisLabelsPositionOnValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'labelsPositionChanged'
        QtMocHelpers::RevisionedSignalData<void(enum AxisLabelsPosition)>(9, 10, QMC::AccessPublic, 0x201, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'append'
        QtMocHelpers::SlotData<void(const QString &, qreal)>(13, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QReal, 15 },
        }}),
        // Slot 'remove'
        QtMocHelpers::SlotData<void(const QString &)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'replace'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(17, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'axisChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'labelsPosition'
        QtMocHelpers::PropertyData<enum AxisLabelsPosition>(21, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0, 0x201),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AxisLabelsPosition'
        QtMocHelpers::EnumData<enum AxisLabelsPosition>(11, 11, QMC::EnumFlags{}).add({
            {   22, AxisLabelsPosition::AxisLabelsPositionCenter },
            {   23, AxisLabelsPosition::AxisLabelsPositionOnValue },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeCategoryAxis, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeCategoryAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QCategoryAxis::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DeclarativeCategoryAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DeclarativeCategoryAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23DeclarativeCategoryAxisE_t>.metaTypes,
    nullptr
} };

void DeclarativeCategoryAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeCategoryAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->labelsPositionChanged((*reinterpret_cast<std::add_pointer_t<enum AxisLabelsPosition>>(_a[1]))); break;
        case 1: _t->append((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 2: _t->remove((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->replace((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeCategoryAxis::*)(AxisLabelsPosition )>(_a, &DeclarativeCategoryAxis::labelsPositionChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->axisChildren(); break;
        case 1: *reinterpret_cast<enum AxisLabelsPosition*>(_v) = _t->labelsPosition(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setLabelsPosition(*reinterpret_cast<enum AxisLabelsPosition*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeCategoryAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeCategoryAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23DeclarativeCategoryAxisE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QCategoryAxis::qt_metacast(_clname);
}

int DeclarativeCategoryAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCategoryAxis::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeCategoryAxis::labelsPositionChanged(AxisLabelsPosition _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
