/****************************************************************************
** Meta object code from reading C++ file 'qpieseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtcharts/src/charts/piechart/qpieseries.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpieseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QPieSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QPieSeries::qt_create_metaobjectdata<qt_meta_tag_ZN10QPieSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QPieSeries",
        "added",
        "",
        "QList<QPieSlice*>",
        "slices",
        "removed",
        "clicked",
        "QPieSlice*",
        "slice",
        "hovered",
        "state",
        "pressed",
        "released",
        "doubleClicked",
        "countChanged",
        "sumChanged",
        "horizontalPosition",
        "verticalPosition",
        "size",
        "startAngle",
        "endAngle",
        "count",
        "sum",
        "holeSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'added'
        QtMocHelpers::SignalData<void(const QList<QPieSlice*> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'removed'
        QtMocHelpers::SignalData<void(const QList<QPieSlice*> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(QPieSlice *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'hovered'
        QtMocHelpers::SignalData<void(QPieSlice *, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Bool, 10 },
        }}),
        // Signal 'pressed'
        QtMocHelpers::SignalData<void(QPieSlice *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'released'
        QtMocHelpers::SignalData<void(QPieSlice *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(QPieSlice *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sumChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'horizontalPosition'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'verticalPosition'
        QtMocHelpers::PropertyData<qreal>(17, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'size'
        QtMocHelpers::PropertyData<qreal>(18, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'startAngle'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'endAngle'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'count'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags, 7),
        // property 'sum'
        QtMocHelpers::PropertyData<qreal>(22, QMetaType::QReal, QMC::DefaultPropertyFlags, 8),
        // property 'holeSize'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QPieSeries, qt_meta_tag_ZN10QPieSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QPieSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QPieSeriesE_t>.metaTypes,
    nullptr
} };

void QPieSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QPieSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->added((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 1: _t->removed((*reinterpret_cast<std::add_pointer_t<QList<QPieSlice*>>>(_a[1]))); break;
        case 2: _t->clicked((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 3: _t->hovered((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->pressed((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 5: _t->released((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 6: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<QPieSlice*>>(_a[1]))); break;
        case 7: _t->countChanged(); break;
        case 8: _t->sumChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPieSlice*> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPieSlice* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<QPieSlice*> & )>(_a, &QPieSeries::added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(const QList<QPieSlice*> & )>(_a, &QPieSeries::removed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::clicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * , bool )>(_a, &QPieSeries::hovered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::pressed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::released, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)(QPieSlice * )>(_a, &QPieSeries::doubleClicked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::countChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QPieSeries::*)()>(_a, &QPieSeries::sumChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->horizontalPosition(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->verticalPosition(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->pieSize(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->pieStartAngle(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->pieEndAngle(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->sum(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->holeSize(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHorizontalPosition(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setVerticalPosition(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setPieSize(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setPieStartAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setPieEndAngle(*reinterpret_cast<qreal*>(_v)); break;
        case 7: _t->setHoleSize(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QPieSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QPieSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QPieSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSeries::qt_metacast(_clname);
}

int QPieSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QPieSeries::added(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QPieSeries::removed(const QList<QPieSlice*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QPieSeries::clicked(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QPieSeries::hovered(QPieSlice * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QPieSeries::pressed(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QPieSeries::released(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QPieSeries::doubleClicked(QPieSlice * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QPieSeries::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QPieSeries::sumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
